from office365.runtime.client_object import ClientObject
from office365.runtime.client_object_collection import ClientObjectCollection
from office365.runtime.resource_path import ResourcePath


class TaxonomyItem(ClientObject):

    def set_property(self, name, value, persist_changes=True):
        super(TaxonomyItem, self).set_property(name, value, persist_changes)
        if self._resource_path is None:
            if name == "id":
                self._resource_path = ResourcePath(value, self._parent_collection.resource_path)
        return self


class TaxonomyItemCollection(ClientObjectCollection):

    def __init__(self, context, taxonomy_item_type, resource_path=None):
        super(TaxonomyItemCollection, self).__init__(context, taxonomy_item_type, resource_path)



from office365.runtime.client_value_collection import ClientValueCollection


class FieldMultiChoiceValue(ClientValueCollection):

    def __init__(self, choices):
        """
        :param list choices: Choice values
        """
        super(FieldMultiChoiceValue, self).__init__(str)
        [self.add(choice) for choice in choices]



from office365.outlook.mail.recipient import Recipient


class AttendeeBase(Recipient):
    """The type of attendee."""

    def __init__(self, emailAddress, attendee_type=None):
        """

        :param office365.mail.emailAddress.EmailAddress emailAddress: Includes the name and SMTP address of the attendee
        :param str attendee_type: The type of attendee. The possible values are: required, optional, resource.
            Currently if the attendee is a person, findMeetingTimes always considers the person is of the Required type.
        """
        super(AttendeeBase, self).__init__(emailAddress)
        self.type = attendee_type



from office365.runtime.client_value import ClientValue


class ListItemCreationInformationUsingPath(ClientValue):

    def __init__(self, leaf_name, object_type, folder_path=None):
        """

        :type leaf_name: str
        :type object_type: int
        :type folder_path: office365.sharepoint.types.resource_path.ResourcePath
        """
        super(ListItemCreationInformationUsingPath, self).__init__()
        self.FolderPath = folder_path
        self.LeafName = leaf_name
        self.UnderlyingObjectType = object_type



from office365.entity import Entity


class OutlookCategory(Entity):
    """epresents a category by which a user can group Outlook items such as messages and events.
    The user defines categories in a master list, and can apply one or more
    of these user-defined categories to an item."""
    pass



from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.webs.web_information import WebInformation


class WebInformationCollection(BaseEntityCollection):
    """Web Information collection"""

    def __init__(self, context, resource_path=None):
        super(WebInformationCollection, self).__init__(context, WebInformation, resource_path)



from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.logger.logFileInfo import LogFileInfo


class LogFileInfoCollection(BaseEntityCollection):

    def __init__(self, context, resource_path=None):
        super(LogFileInfoCollection, self).__init__(context, LogFileInfo, resource_path)



from office365.runtime.client_value import ClientValue


class TeamGuestSettings(ClientValue):
    """Settings to configure whether guests can create, update, or delete channels in the team."""

    def __init__(self, allow_create_update_channels=True, allow_delete_channels=True):
        super(TeamGuestSettings, self).__init__()
        self.allowCreateUpdateChannels = allow_create_update_channels
        self.allowDeleteChannels = allow_delete_channels



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.webs.timeZoneInformation import TimeZoneInformation


class TimeZone(BaseEntity):
    """Represents the time zone setting that is implemented on a SharePoint Web site."""

    def local_time_to_utc(self, date):
        """
        Converts the specified date from local time to Coordinated Universal Time (UTC).

        :param datetime.datetime date: The local date and time value to convert.
        :return:
        """
        result = ClientResult(self.context)
        qry = ServiceOperationQuery(self, "LocalTimeToUTC", [date], None, None, result)
        self.context.add_query(qry)
        return result

    @property
    def id(self):
        """Gets the identifier of the time zone."""
        return self.properties.get('Id', None)

    @property
    def description(self):
        """Gets the description of the time zone."""
        return self.properties.get('Description', None)

    @property
    def information(self):
        """Gets information about the time zone."""
        return self.properties.get('Information', TimeZoneInformation())


class TimeZoneCollection(BaseEntityCollection):
    """TimeZone collection"""

    def __init__(self, context, resource_path=None):
        super(TimeZoneCollection, self).__init__(context, TimeZone, resource_path)




from office365.directory.licenses.service_plan_info import ServicePlanInfo
from office365.entity import Entity
from office365.runtime.client_value_collection import ClientValueCollection


class LicenseDetails(Entity):
    """Contains information about a license assigned to a user."""

    @property
    def service_plans(self):
        """
        Information about the service plans assigned with the license. Read-only, Not nullable
        """
        return self.properties.get('servicePlans', ClientValueCollection(ServicePlanInfo))

    @property
    def sku_id(self):
        """
        Unique identifier (GUID) for the service SKU. Equal to the skuId property on the related SubscribedSku object.
        Read-only
        :rtype: str or None
        """
        return self.properties.get('skuId', None)

    @property
    def sku_part_number(self):
        """
        Unique SKU display name. Equal to the skuPartNumber on the related SubscribedSku object;
        for example: "AAD_Premium". Read-only
        :rtype: str or None
        """
        return self.properties.get('skuPartNumber', None)



from office365.entity_collection import EntityCollection
from office365.runtime.compat import quote

from office365.entity import Entity
from office365.onedrive.driveitems.driveItem import DriveItem
from office365.runtime.resource_path import ResourcePath
from office365.teams.channels.conversation_member import ConversationMember
from office365.teams.messages.chat_message import ChatMessage
from office365.teams.tabs.teams_tab import TeamsTab


class Channel(Entity):
    """Teams are made up of channels, which are the conversations you have with your teammates"""

    @property
    def files_folder(self):
        """Get the metadata for the location where the files of a channel are stored."""
        return self.properties.get('filesFolder',
                                   DriveItem(self.context, ResourcePath("filesFolder", self.resource_path)))

    @property
    def tabs(self):
        """A collection of all the tabs in the channel. A navigation property."""
        return self.properties.get('tabs',
                                   EntityCollection(self.context, TeamsTab, ResourcePath("tabs", self.resource_path)))

    @property
    def messages(self):
        """
        A collection of all the messages in the channel. A navigation property. Nullable.
        """
        return self.properties.get('messages',
                                   EntityCollection(self.context, ChatMessage,
                                                    ResourcePath("messages", self.resource_path)))

    @property
    def members(self):
        """A collection of membership records associated with the channel.

        :rtype: EntityCollection
        """
        return self.get_property('members',
                                 EntityCollection(self.context, ConversationMember,
                                                  ResourcePath("members", self.resource_path)))

    @property
    def web_url(self):
        """A hyperlink that will navigate to the channel in Microsoft Teams. This is the URL that you get when you
        right-click a channel in Microsoft Teams and select Get link to channel. This URL should be treated as an
        opaque blob, and not parsed. Read-only.

        :rtype: str or None """
        return self.properties.get('webUrl', None)

    def set_property(self, name, value, persist_changes=True):
        super(Channel, self).set_property(name, value, persist_changes)
        # fallback: fix resource path
        if name == "id":
            channel_id = quote(value)
            self._resource_path = ResourcePath(channel_id, self.resource_path.parent)
        return self



from office365.runtime.client_value import ClientValue


class ContextWebInformation(ClientValue):
    """The context information for a site."""

    def __init__(self):
        super(ContextWebInformation, self).__init__()
        self.FormDigestValue = None
        self.FormDigestTimeoutSeconds = None
        self.LibraryVersion = None
        self.SiteFullUrl = None
        self.SupportedSchemaVersions = None
        self.WebFullUrl = None



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.navigation.menu_state import MenuState


class NavigationService(BaseEntity):

    def __init__(self, context):
        """The entry point for REST-based navigation service operations."""
        super(NavigationService, self).__init__(context, ResourcePath("Microsoft.SharePoint.Navigation.REST.NavigationServiceRest"))

    def get_publishing_navigation_provider_type(self, mapProviderName="SPNavigationProvider"):
        """
        Gets a publishing navigation provider type when publishing feature is turned on for the site (2).
        If navigation provider is not found on the site MUST return InvalidSiteMapProvider type.

        :param str mapProviderName: The server will use "SPNavigationProvider" as provider name
            if mapProviderName is not specified.
        :return:
        """
        result = ClientResult(self.context)
        params = {"mapProviderName": mapProviderName}
        qry = ServiceOperationQuery(self, "GetPublishingNavigationProviderType", params, None, None, result)
        self.context.add_query(qry)
        return result

    def global_nav(self):
        return_type = MenuState()
        qry = ServiceOperationQuery(self, "GlobalNav", None, None, None, return_type)
        self.context.add_query(qry)
        return return_type

    def global_nav_enabled(self):
        result = ClientResult(self.context)
        qry = ServiceOperationQuery(self, "GlobalNavEnabled", None, None, None, result)
        self.context.add_query(qry)
        return result

    def set_global_nav_enabled(self, is_enabled):
        """
        :param bool is_enabled:
        :return:
        """
        qry = ServiceOperationQuery(self, "SetGlobalNavEnabled", None, {"isEnabled": is_enabled}, None)
        self.context.add_query(qry)
        return self

    def menu_node_key(self, currentUrl):
        pass

    def save_menu_state(self):
        pass



from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.publishing.video_item import VideoItem


class VideoItemCollection(BaseEntityCollection):

    def __init__(self, context, resource_path=None):
        super(VideoItemCollection, self).__init__(context, VideoItem, resource_path)



from office365.directory.users.user import User
from office365.entity_collection import EntityCollection
from office365.runtime.queries.create_entity_query import CreateEntityQuery


class UserCollection(EntityCollection):
    """User's collection"""

    def __init__(self, context, resource_path=None):
        super(UserCollection, self).__init__(context, User, resource_path)

    def add(self, user_properties):
        """Create a new user.

        :type user_properties: UserProfile
        """
        usr = User(self.context)
        qry = CreateEntityQuery(self, user_properties, usr)
        self.context.add_query(qry)
        self.add_child(usr)
        return usr



from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.base_entity_collection import BaseEntityCollection


class SiteCollectionCorporateCatalogAccessor(BaseEntity):
    pass


class SiteCollectionAppCatalogAllowedItem(BaseEntity):
    pass


class SiteCollectionAppCatalogAllowedItems(BaseEntityCollection):

    def __init__(self, context, resource_path=None):
        super(SiteCollectionAppCatalogAllowedItems, self).__init__(context, SiteCollectionAppCatalogAllowedItem,
                                                                   resource_path)



from office365.entity_collection import EntityCollection
from office365.outlook.attachment_upload_query import AttachmentUploadQuery
from office365.outlook.mail.attachment import Attachment
from office365.onedrive.uploadSession import UploadSession
from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery


class AttachmentCollection(EntityCollection):
    """Attachment collection"""

    def __init__(self, context, resource_path=None):
        super(AttachmentCollection, self).__init__(context, Attachment, resource_path)

    def resumable_upload(self, source_path, chunk_size=1000000):
        """
        Create an upload session to allow your app to upload files up to the maximum file size.
        An upload session allows your app to upload ranges of the file in sequential API requests,
        which allows the transfer to be resumed if a connection is dropped while the upload is in progress.

        :param str source_path: Local file path
        :param int chunk_size: chunk size
        """
        upload_query = AttachmentUploadQuery(self, source_path, chunk_size)
        self.context.add_query(upload_query)
        return upload_query.return_type

    def create_upload_session(self, attachment_item):
        """
        Create an upload session that allows an app to iteratively upload ranges of a file,
        so as to attach the file to the specified Outlook item. The item can be a message or event.
        :type attachment_item: office365.mail.attachment_item.AttachmentItem
        """
        result = ClientResult(self.context, UploadSession())
        qry = ServiceOperationQuery(self,
                                    "createUploadSession",
                                    None,
                                    {
                                        "AttachmentItem": attachment_item
                                    },
                                    None,
                                    result
                                    )
        self.context.add_query(qry)
        return result



from office365.entity import Entity
from office365.entity_collection import EntityCollection
from office365.runtime.resource_path import ResourcePath
from office365.teams.apps.teams_app_installation import TeamsAppInstallation
from office365.teams.channels.conversation_member import ConversationMember
from office365.teams.messages.chat_message import ChatMessage
from office365.teams.operations.teams_async_operation import TeamsAsyncOperation


class Chat(Entity):
    """A chat is a collection of chatMessages between one or more participants. Participants can be users or apps."""

    @property
    def topic(self):
        """(Optional) Subject or topic for the chat. Only available for group chats."""
        return self.properties.get("topic", None)

    @property
    def installed_apps(self):
        """A collection of all the apps in the chat. Nullable.

        :rtype: EntityCollection
        """
        return self.get_property('installedApps',
                                 EntityCollection(self.context, TeamsAppInstallation,
                                                  ResourcePath("installedApps", self.resource_path)))

    @property
    def members(self):
        """A collection of membership records associated with the chat.

        :rtype: EntityCollection
        """
        return self.get_property('members',
                                 EntityCollection(self.context, ConversationMember,
                                                  ResourcePath("members", self.resource_path)))

    @property
    def messages(self):
        """A collection of all the messages in the chat. Nullable."""
        return self.properties.get('messages', EntityCollection(self.context, ChatMessage,
                                                                ResourcePath("messages", self.resource_path)))

    @property
    def operations(self):
        """
        A collection of all the Teams async operations that ran or are running on the chat. Nullable.

        :rtype: EntityCollection
        """
        return self.get_property('operations',
                                 EntityCollection(self.context, TeamsAsyncOperation,
                                                  ResourcePath("operations", self.resource_path)))



from office365.outlook.calendar.attendee_base import AttendeeBase
from office365.outlook.calendar.emailAddress import EmailAddress


class Attendee(AttendeeBase):
    """An event attendee. This can be a person or resource such as a meeting room or equipment,
    that has been set up as a resource on the Exchange server for the tenant."""

    def __init__(self, emailAddress=EmailAddress(), attendee_type=None, proposedNewTime=None, status=None):
        """

        :param office365.mail.emailAddress.EmailAddress emailAddress emailAddress:
        :param office365.calendar.timeSlot.TimeSlot proposedNewTime:
        :param str status: The attendee's response (none, accepted, declined, etc.) for the event and date-time
            that the response was sent.
        """
        super(Attendee, self).__init__(emailAddress, attendee_type)
        self.proposedNewTime = proposedNewTime
        self.status = status



from office365.base_item import BaseItem
from office365.entity_collection import EntityCollection
from office365.onedrive.driveitems.driveItem import DriveItem
from office365.onedrive.lists.list import List
from office365.onedrive.internal.root_resource_path import RootResourcePath
from office365.onedrive.system_facet import SystemFacet
from office365.runtime.resource_path import ResourcePath


class Drive(BaseItem):
    """The drive resource is the top level object representing a user's OneDrive or a document library in
    SharePoint. """

    @property
    def system(self):
        """If present, indicates that this is a system-managed drive. Read-only."""
        return self.properties.get('system', SystemFacet())

    @property
    def shared_with_me(self):
        """Retrieve a collection of DriveItem resources that have been shared with the owner of the Drive.

        :rtype: EntityCollection
        """
        return self.get_property('sharedWithMe',
                                 EntityCollection(self.context, DriveItem,
                                                  ResourcePath("sharedWithMe", self.resource_path)))

    @property
    def root(self):
        """The root folder of the drive.

        :rtype: DriveItem
        """
        return self.get_property('root', DriveItem(self.context, RootResourcePath(self.resource_path)))

    @property
    def list(self):
        """For drives in SharePoint, the underlying document library list.

        :rtype: List
        """
        return self.get_property('list', List(self.context, ResourcePath("list", self.resource_path)))

    @property
    def items(self):
        """All items contained in the drive.

        :rtype: EntityCollection
        """
        return self.get_property('items',
                                 EntityCollection(self.context, DriveItem, ResourcePath("items", self.resource_path)))

    @property
    def following(self):
        """The list of items the user is following. Only in OneDrive for Business.

        :rtype: EntityCollection
        """
        return self.get_property('following',
                                 EntityCollection(self.context, DriveItem,
                                                  ResourcePath("following", self.resource_path)))

    @property
    def special(self):
        """Collection of common folders available in OneDrive. Read-only. Nullable.

        :rtype: EntityCollection
        """
        return self.get_property('special',
                                 EntityCollection(self.context, DriveItem,
                                                  ResourcePath("special", self.resource_path)))



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.permissions.base_permissions import BasePermissions
from office365.sharepoint.permissions.role_assignment import RoleAssignment
from office365.sharepoint.permissions.roleAssignmentCollection import RoleAssignmentCollection


class SecurableObject(BaseEntity):
    """An object that can be assigned security permissions."""

    def get_role_assignment(self, principal):
        """

        :param office365.sharepoint.principal.principal.Principal principal: Specifies the user or group of the
        role assignment.
        :return: RoleAssignment
        """
        result = ClientResult(self.context, RoleAssignment(self.context))

        def _principal_loaded():
            result.value = self.roleAssignments.get_by_principal_id(principal.id)
        principal.ensure_property("Id", _principal_loaded)
        return result.value

    def add_role_assignment(self, principal, role_def):
        """Adds a role assignment to securable resource.<81>

        :param office365.sharepoint.permissions.role_definition.RoleDefinition role_def: Specifies the role definition
        of the role assignment.
        :param office365.sharepoint.principal.principal.Principal principal: Specifies the user or group of the
        role assignment.
        """

        def _principal_loaded():
            role_def.ensure_property("Id", _role_def_loaded)

        def _role_def_loaded():
            self.roleAssignments.add_role_assignment(principal.id, role_def.id)

        principal.ensure_property("Id", _principal_loaded)
        return self

    def remove_role_assignment(self, principal, role_def):
        """Removes a role assignment from a securable resource.<81>

        :param office365.sharepoint.permissions.role_definition.RoleDefinition role_def: Specifies the role definition
        of the role assignment.
        :param office365.sharepoint.principal.principal.Principal principal: Specifies the user or group of the
        role assignment.
        """

        def _principal_loaded():
            def _role_def_loaded():
                self.roleAssignments.remove_role_assignment(principal.id, role_def.id)
            role_def.ensure_property("Id", _role_def_loaded)

        principal.ensure_property("Id", _principal_loaded)
        return self

    def break_role_inheritance(self, copyRoleAssignments=True, clearSubscopes=True):
        """Creates unique role assignments for the securable object. If the securable object already has
        unique role assignments, the protocol server MUST NOT alter any role assignments.

        :param bool clearSubscopes:  If the securable object is a site (2), and the clearSubscopes parameter is "true",
        the role assignments for all child securable objects in the current site (2) and in the sites (2) that inherit
        role assignments from the current site (2) MUST be cleared and those securable objects inherit role assignments
        from the current site (2) after this call. If the securable object is a site (2), and the clearSubscopes
        parameter is "false", the role assignments for all child securable objects that do not inherit role assignments
        from their parent object (1) MUST remain unchanged. If the securable object is not a site (2), and the
        clearSubscopes parameter is "true", the role assignments for all child securable objects MUST be cleared and
        those securable objects inherit role assignments from the current securable object after this call. If the
        securable object is not a site (2), and the clearSubscopes parameter is "false", the role assignments for all
        child securable objects that do not inherit role assignments from their parent object (1) MUST remain unchanged.
        :param bool copyRoleAssignments: Specifies whether to copy the role assignments from
        the parent securable object.If the value is "false", the collection of role assignments MUST contain
        only 1 role assignment containing the current user after the operation.

        """
        payload = {
            "copyRoleAssignments": copyRoleAssignments,
            "clearSubscopes": clearSubscopes
        }
        qry = ServiceOperationQuery(self, "breakRoleInheritance", None, payload, None, None)
        self.context.add_query(qry)
        return self

    def reset_role_inheritance(self):
        """Resets the role inheritance for the securable object and inherits role assignments from
        the parent securable object."""
        qry = ServiceOperationQuery(self, "resetRoleInheritance", None, None, None, None)
        self.context.add_query(qry)
        return self

    def get_user_effective_permissions(self, user_name):
        """
        Returns the user permissions for this list.

        :param str user_name: Specifies the user login name.
        """
        result = ClientResult(self.context, BasePermissions())
        qry = ServiceOperationQuery(self, "getUserEffectivePermissions", [user_name], None, None, result)
        self.context.add_query(qry)
        return result

    @property
    def has_unique_role_assignments(self):
        """Specifies whether the role assignments are uniquely defined for this securable object or inherited from a
        parent securable object. If the value is "false", role assignments are inherited from a parent securable
        object.

        :rtype: bool or None
        """
        return self.properties.get("HasUniqueRoleAssignments", None)

    @property
    def firstUniqueAncestorSecurableObject(self):
        """Specifies the object where role assignments for this object are defined.<85>."""
        return self.properties.get("FirstUniqueAncestorSecurableObject",
                                   SecurableObject(self.context,
                                                   ResourcePath("FirstUniqueAncestorSecurableObject",
                                                                self.resource_path)))

    @property
    def roleAssignments(self):
        """The role assignments for the securable object."""
        return self.properties.get("RoleAssignments",
                                   RoleAssignmentCollection(self.context,
                                                            ResourcePath("RoleAssignments", self.resource_path)))



from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.tenant.administration.hub_site import HubSite


class HubSiteCollection(BaseEntityCollection):
    """Represents a collection of HubSite resources."""

    def __init__(self, context, resource_path=None):
        super(HubSiteCollection, self).__init__(context, HubSite, resource_path)



from office365.runtime.client_value import ClientValue


class DriveItemUploadableProperties(ClientValue):
    """The driveItemUploadableProperties resource represents an item being uploaded when creating an upload session."""

    def __init__(self):
        super(DriveItemUploadableProperties, self).__init__()
        self.fileSystemInfo = None
        self.name = None
        self.description = None
        self.__fileSize = None

    @property
    def file_size(self):
        """Provides an expected file size to perform a quota check prior to upload. Only on OneDrive Personal."""
        return self.__fileSize



from office365.entity import Entity


class CommsOperation(Entity):
    """
    Represents the status of certain long-running operations.

    This resource can be returned as the response to an action, or as the content of a commsNotification.

    When it is returned as a response to an action, the status indicates whether there will be subsequent notifications.
    If, for example, an operation with status of completed or failed is returned, there will not be any subsequent
    operation via the notification channel.

    If a null operation, or an operation with a status of notStarted or running is returned, subsequent updates will
    come via the notification channel.
    """
    pass



from office365.entity import Entity
from office365.entity_collection import EntityCollection
from office365.excel.functions.workbookFunctions import WorkbookFunctions
from office365.excel.names.workbookNamedItem import WorkbookNamedItem
from office365.excel.tables.workbook_table import WorkbookTable
from office365.excel.worksheets.workbookWorksheet import WorkbookWorksheetCollection
from office365.runtime.resource_path import ResourcePath


class Workbook(Entity):
    """The top-level object that contains related workbook objects such as worksheets, tables, and ranges."""

    @property
    def functions(self):
        return self.properties.get('functions',
                                   WorkbookFunctions(self.context, ResourcePath("functions", self.resource_path)))

    @property
    def tables(self):
        """Represents a collection of tables associated with the workbook. Read-only."""
        return self.properties.get('tables',
                                   EntityCollection(self.context, WorkbookTable,
                                                    ResourcePath("tables", self.resource_path)))

    @property
    def names(self):
        """Represents a collection of workbook scoped named items (named ranges and constants). Read-only."""
        return self.properties.get('names',
                                   EntityCollection(self.context, WorkbookNamedItem,
                                                    ResourcePath("names", self.resource_path)))

    @property
    def worksheets(self):
        """Represents a collection of worksheets associated with the workbook. Read-only."""
        return self.properties.get('worksheets',
                                   WorkbookWorksheetCollection(self.context,
                                                               ResourcePath("worksheets", self.resource_path)))



from office365.runtime.client_result import ClientResult
from office365.runtime.client_value import ClientValue
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.principal.principal_source import PrincipalSource
from office365.sharepoint.principal.principal_type import PrincipalType
from office365.sharepoint.ui.applicationpages.picker_entity_types import PickerEntityInformation


class PeoplePickerQuerySettings(ClientValue):
    pass


class ClientPeoplePickerQueryParameters(ClientValue):

    def __init__(self, queryString, allowEmailAddresses=True, allowMultipleEntities=True, allowOnlyEmailAddresses=False,
                 allUrlZones=False, enabledClaimProviders=None, forceClaims=False, maximumEntitySuggestions=1,
                 principalSource=PrincipalSource.All, principalType=PrincipalType.All, urlZone=0,
                 urlZoneSpecified=False, sharePointGroupID=0):
        """
        Specifies the properties of a principal query

        :type int urlZone: Specifies a location in the topology of the farm for the principal query.
        :param int sharePointGroupID: specifies a group containing allowed principals to be used in the principal query.
        :param str queryString: Specifies the value to be used in the principal query.
        :param int principalType: Specifies the type to be used in the principal query.
        :param int principalSource: Specifies the source to be used in the principal query.
        :param int maximumEntitySuggestions: Specifies the maximum number of principals to be returned by the
        principal query.
        :param bool forceClaims: Specifies whether the principal query SHOULD be handled by claims providers.
        :param bool enabledClaimProviders: Specifies the claims providers to be used in the principal query.
        :param bool allUrlZones: Specifies whether the principal query will search all locations in the topology
        of the farm.
        :param bool allowOnlyEmailAddresses: Specifies whether to allow the picker to resolve only email addresses as
        valid entities. This property is only used when AllowEmailAddresses (section 3.2.5.217.1.1.1) is set to True.
        Otherwise it is ignored.
        :param bool allowMultipleEntities: Specifies whether the principal query allows multiple values.
        :param bool allowEmailAddresses: Specifies whether the principal query can return a resolved principal
        matching an unverified e-mail address when unable to resolve to a known principal.
        """
        super(ClientPeoplePickerQueryParameters, self).__init__()
        self.QueryString = queryString
        self.AllowEmailAddresses = allowEmailAddresses
        self.AllowMultipleEntities = allowMultipleEntities
        self.AllowOnlyEmailAddresses = allowOnlyEmailAddresses
        self.AllUrlZones = allUrlZones
        self.EnabledClaimProviders = enabledClaimProviders
        self.ForceClaims = forceClaims
        self.MaximumEntitySuggestions = maximumEntitySuggestions
        self.PrincipalSource = principalSource
        self.PrincipalType = principalType
        self.UrlZone = urlZone
        self.UrlZoneSpecified = urlZoneSpecified
        self.SharePointGroupID = sharePointGroupID

    @property
    def entity_type_name(self):
        return "SP.UI.ApplicationPages.ClientPeoplePickerQueryParameters"


class ClientPeoplePickerWebServiceInterface(BaseEntity):

    def __init__(self, context):
        super(ClientPeoplePickerWebServiceInterface, self).__init__(context)

    @staticmethod
    def get_search_results(context, searchPattern, providerID=None, hierarchyNodeID=None, entityTypes=None):
        """
        :type context: office365.sharepoint.client_context.ClientContext
        :type searchPattern: str
        :type providerID: str
        :type hierarchyNodeID: str
        :type entityTypes: str
        """
        result = ClientResult(context)
        payload = {
            "searchPattern": searchPattern,
            "providerID": providerID,
            "hierarchyNodeID": hierarchyNodeID,
            "entityTypes": entityTypes
        }
        svc = ClientPeoplePickerWebServiceInterface(context)
        qry = ServiceOperationQuery(svc, "GetSearchResults", None, payload, None, result)
        qry.static = True
        context.add_query(qry)
        return result

    @staticmethod
    def client_people_picker_resolve_user(context, query_params, on_resolved=None):
        """
        Resolves the principals to a string of JSON representing users in people picker format.


        :param (str) -> None on_resolved: resolved event
        :param ClientPeoplePickerQueryParameters query_params: Specifies the properties of a principal query.
        :param office365.sharepoint.client_context.ClientContext context:

        """
        result = ClientResult(context)
        svc = ClientPeoplePickerWebServiceInterface(context)
        qry = ServiceOperationQuery(svc, "ClientPeoplePickerResolveUser", None, query_params, "queryParams", result)
        qry.static = True
        context.add_query(qry)

        def _process_result(resp):
            result.value = "[{0}]".format(result.value)
            if callable(on_resolved):
                on_resolved(result.value)

        context.after_execute(_process_result)
        return result

    @staticmethod
    def get_picker_entity_information(context, request):
        """

        :param office365.sharepoint.client_context.ClientContext context:
        :param office365.sharepoint.ui.applicationpages.picker_entity_types.PickerEntityInformationRequest request:

        """
        result = ClientResult(context, PickerEntityInformation(context))
        svc = ClientPeoplePickerWebServiceInterface(context)
        qry = ServiceOperationQuery(svc, "GetPickerEntityInformation",
                                    None,
                                    request,
                                    "entityInformationRequest",
                                    result)
        qry.static = True
        context.add_query(qry)
        return result

    @property
    def entity_type_name(self):
        return "SP.UI.ApplicationPages.ClientPeoplePickerWebServiceInterface"



from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.userprofiles.followedContent import FollowedContent


class UserProfile(BaseEntity):

    @property
    def public_url(self):
        """
        :rtype: str or None
        """
        return self.properties.get("PublicUrl", None)

    @property
    def followed_content(self):
        """
        Gets a FollowedContent object for the user.
        """
        return self.properties.get("FollowedContent",
                                   FollowedContent(self.context, ResourcePath("FollowedContent", self.resource_path)))

    def create_personal_site_enque(self, isInteractive):
        """
        Enqueues creating a personal site for this user, which can be used to share documents, web pages,
            and other files.

        :type isInteractive: bool
        """
        payload = {"isInteractive": isInteractive}
        qry = ServiceOperationQuery(self, "CreatePersonalSiteEnque", None, payload, None, None)
        self.context.add_query(qry)
        return self



from office365.base_item import BaseItem
from office365.entity_collection import EntityCollection
from office365.onedrive.columns.column_link import ColumnLink
from office365.runtime.resource_path import ResourcePath


class ContentType(BaseItem):
    """The contentType resource represents a content type in SharePoint. Content types allow you to define a set of
    columns that must be present on every listItem in a list. """

    @property
    def name(self):
        return self.properties.get('name', None)

    @property
    def parent_id(self):
        return self.properties.get('parentId', None)

    @property
    def read_only(self):
        return self.properties.get('readOnly', None)

    @property
    def column_links(self):
        return self.properties.get('columnLinks',
                                   EntityCollection(self.context,
                                                    ColumnLink,
                                                    ResourcePath("columnLinks", self.resource_path)))



from office365.runtime.client_value import ClientValue


class PasswordProfile(ClientValue):
    """Contains the password profile associated with a user. The passwordProfile property of the user entity is a
    passwordProfile object. """
    def __init__(self, password):
        super(PasswordProfile, self).__init__()
        self.password = password
        self.forceChangePasswordNextSignIn = True



import inspect

from office365.runtime.client_value import ClientValue
from office365.sharepoint.permissions.permission_kind import PermissionKind


class BasePermissions(ClientValue):
    """Specifies a set of built-in permissions."""

    def __init__(self):
        super(BasePermissions, self).__init__()
        self.High = 0
        self.Low = 0

    def set(self, perm):
        """

        :type perm: int
        """
        if perm == PermissionKind.FullMask:
            self.Low = 65535
            self.High = 65535
        elif perm == PermissionKind.EmptyMask:
            self.Low = 0
            self.High = 0
        else:
            high = perm - 1
            low = 1
            if 0 <= high < 32:
                self.Low |= low << high
            else:
                if high < 32 or high >= 64:
                    return
                self.High |= low << high - 32

    def has(self, perm):
        """Determines whether the current instance has the specified permission.

        """
        if perm == PermissionKind.EmptyMask:
            return True
        if perm == PermissionKind.FullMask:
            if int(self.High) & 32767 == 32767:
                return int(self.Low) == 65535
            return False
        high = perm - 1
        low = 1
        if 0 <= high < 32:
            return 0 != (int(self.Low) & (low << high))
        if 32 <= high < 64:
            return 0 != (int(self.High) & (low << high - 32))
        return False

    def clear_all(self):
        """Clears all permissions for the current instance."""
        self.Low = 0
        self.High = 0

    def to_json(self):
        return {'Low': str(self.High), 'High': str(self.Low)}

    @property
    def permission_levels(self):
        result = []
        for k, v in inspect.getmembers(PermissionKind):
            if isinstance(v, int) and self.has(v):
                result.append(k)
        return result



from office365.sharepoint.fields.field import Field


class FieldComputed(Field):

    @property
    def enable_lookup(self):
        """
        Specifies whether a lookup field can reference the field (2).

        :rtype: bool or None
        """
        return self.properties.get("EnableLookup", None)



from office365.base_item import BaseItem
from office365.entity_collection import EntityCollection
from office365.onedrive.contenttypes.content_type_info import ContentTypeInfo
from office365.onedrive.listitems.fieldValueSet import FieldValueSet
from office365.onedrive.itemAnalytics import ItemAnalytics
from office365.onedrive.listitems.listItemVersion import ListItemVersion
from office365.runtime.resource_path import ResourcePath


class ListItem(BaseItem):
    """Represents an item in a SharePoint list. Column values in the list are available through the fieldValueSet
    dictionary. """

    @property
    def fields(self):
        """The values of the columns set on this list item."""
        return self.properties.get('fields',
                                   FieldValueSet(self.context, ResourcePath("fields", self.resource_path)))

    @property
    def versions(self):
        """The list of previous versions of the list item."""
        return self.properties.get('versions',
                                   EntityCollection(self.context, ListItemVersion,
                                                    ResourcePath("versions", self.resource_path)))

    @property
    def drive_item(self):
        """For document libraries, the driveItem relationship exposes the listItem as a driveItem."""
        from office365.onedrive.driveitems.driveItem import DriveItem
        return self.properties.get('driveItem', DriveItem(self.context, ResourcePath("driveItem", self.resource_path)))

    @property
    def content_type(self):
        """The content type of this list item"""
        return self.properties.get("contentType", ContentTypeInfo())

    @property
    def analytics(self):
        """Analytics about the view activities that took place on this item."""
        return self.properties.get('analytics',
                                   ItemAnalytics(self.context, ResourcePath("analytics", self.resource_path)))




from office365.runtime.client_value import ClientValue


class ChangeLogItemQuery(ClientValue):

    def __init__(self, change_token=None, contains=None, row_limit=None):
        """

        :type change_token: str
        :type contains: str
        :type row_limit: str
        """
        super(ChangeLogItemQuery, self).__init__()
        self.ChangeToken = change_token
        self.Contains = contains
        self.RowLimit = row_limit

    @property
    def entity_type_name(self):
        return 'SP.ChangeLogItemQuery'



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.administration.org_assets import OrgAssets
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.files.file import File
from office365.sharepoint.publishing.file_picker_options import FilePickerOptions
from office365.sharepoint.publishing.primary_city_time import PrimaryCityTime
from office365.sharepoint.publishing.site_page_metadata_collection import SitePageMetadataCollection


class SitePageService(BaseEntity):

    def __init__(self, context):
        """Represents a set of APIs to use for managing site pages."""
        super(SitePageService, self).__init__(context, ResourcePath("SP.Publishing.SitePageService"))

    def pages(self):
        return self.properties.get("pages",
                                   SitePageMetadataCollection(self.context, ResourcePath("pages", self.resource_path)))

    @property
    def entity_type_name(self):
        return "SP.Publishing.SitePageService"

    @staticmethod
    def get_time_zone(context, city_name):
        """
        Gets time zone data for specified city.

        :param office365.sharepoint.client_context.ClientContext context:
        :param str city_name: The name of the city.
        :return: PrimaryCityTime
        """
        return_type = PrimaryCityTime(context)
        svc = SitePageService(context)
        params = {"cityName": city_name}
        qry = ServiceOperationQuery(svc, "GetTimeZone", params, None, None, return_type)
        qry.static = True
        context.add_query(qry)
        return return_type

    @staticmethod
    def compute_file_name(context, title):
        """

        :param office365.sharepoint.client_context.ClientContext context: Client context
        :param str title: The title of the page.
        """
        return_type = ClientResult(context)
        svc = SitePageService(context)
        params = {"title": title}
        qry = ServiceOperationQuery(svc, "ComputeFileName", params, None, None, return_type)
        qry.static = True
        context.add_query(qry)
        return return_type

    @staticmethod
    def get_available_full_page_applications():
        pass

    @staticmethod
    def get_current_user_memberships():
        pass

    @staticmethod
    def is_file_picker_external_image_search_enabled():
        pass

    @staticmethod
    def org_assets(context):
        """

        :param office365.sharepoint.client_context.ClientContext context: Client context
        """
        result = ClientResult(context, OrgAssets())
        svc = SitePageService(context)
        qry = ServiceOperationQuery(svc, "OrgAssets", None, None, None, result)
        qry.static = True
        context.add_query(qry)
        return result

    @staticmethod
    def file_picker_tab_options(context):
        """

        :param office365.sharepoint.client_context.ClientContext context: Client context
        """
        result = ClientResult(context, FilePickerOptions())
        svc = SitePageService(context)
        qry = ServiceOperationQuery(svc, "FilePickerTabOptions", None, None, None, result)
        qry.static = True
        context.add_query(qry)
        return result

    def add_image(self, page_name, image_file_name, image_stream):
        """
        Adds an image to the site assets library of the current web.
        Returns a File object ([MS-CSOMSPT] section 3.2.5.64) that represents the image.

        :param str image_stream: The image stream.
        :param str image_file_name: Indicates the file name of the image to be added.
        :param str page_name: Indicates the name of that site page that the image is to be used in.
        :return: File
        """
        return_type = File(self.context)
        params = {"pageName": page_name, "imageFileName": image_file_name, "imageStream": image_stream}
        qry = ServiceOperationQuery(self, "AddImage", params, None, None, return_type)
        qry.static = True
        self.context.add_query(qry)
        return return_type



from office365.runtime.client_value import ClientValue


class SecondaryAdministratorsInfo(ClientValue):

    def __init__(self, email=None, loginName=None, userPrincipalName=None):
        """

        :param str email:
        :param str loginName:
        :param str userPrincipalName:
        """
        super(SecondaryAdministratorsInfo, self).__init__()
        self.email = email
        self.loginName = loginName
        self.userPrincipalName = userPrincipalName



from office365.directory.directory_object import DirectoryObject
from office365.directory.keyCredential import KeyCredential
from office365.directory.password_credential import PasswordCredential
from office365.runtime.client_result import ClientResult
from office365.runtime.client_value_collection import ClientValueCollection
from office365.runtime.queries.service_operation_query import ServiceOperationQuery


class Application(DirectoryObject):
    """
    Represents an application. Any application that outsources authentication to Azure Active Directory (Azure AD)
    must be registered in a directory. Application registration involves telling Azure AD about your application,
    including the URL where it's located, the URL to send replies after authentication,
    the URI to identify your application, and more. For more information, see Basics of Registering
    an Application in Azure AD
    """

    def add_password(self, display_name):
        """Adds a strong password to an application.
        :param str display_name: App display name
        """
        params = PasswordCredential(displayName=display_name)
        result = ClientResult(self.context, params)
        qry = ServiceOperationQuery(self, "addPassword", None, params, None, result)
        self.context.add_query(qry)
        return result

    def remove_password(self, key_id):
        """Remove a password from an application."""
        qry = ServiceOperationQuery(self, "removePassword", None, {"keyId": key_id})
        self.context.add_query(qry)
        return self

    def delete_object(self, permanent_delete=False):
        """
        :param permanent_delete: Permanently deletes the application from directory
        :type permanent_delete: bool

        """
        super(Application, self).delete_object()
        if permanent_delete:
            deleted_item = self.context.directory.deleted_applications[self.id]
            deleted_item.delete_object()
        return self

    @property
    def key_credentials(self):
        """The collection of key credentials associated with the application. Not nullable.
        """
        return self.properties.get('keyCredentials', ClientValueCollection(KeyCredential))



class EventHandler:
    def __init__(self, once=False):
        self._listeners = []
        self._once = once

    def __iadd__(self, listener):
        self._listeners.append(listener)
        return self

    def __isub__(self, listener):
        self._listeners.remove(listener)
        return self

    def notify(self, *args, **kwargs):
        for listener in self._listeners:
            if self._once:
                self._listeners.remove(listener)
            listener(*args, **kwargs)



import datetime
from datetime import datetime, timedelta
from office365.runtime.compat import urlparse, timezone


class STSProfile(object):

    def __init__(self, authority_url):
        """

        :type authority_url: str
        """
        self.authorityUrl = authority_url
        self.serviceUrl = 'https://login.microsoftonline.com'
        self.securityTokenServicePath = 'extSTS.srf'
        self.userRealmServicePath = 'GetUserRealm.srf'
        self.tokenIssuer = 'urn:federation:MicrosoftOnline'
        now = datetime.now(tz=timezone.utc)
        self.created = now.astimezone(timezone.utc).isoformat('T')[:-9] + 'Z'
        self.expires = (now + timedelta(minutes=10)).astimezone(timezone.utc).isoformat('T')[:-9] + 'Z'
        self.signInPage = '_forms/default.aspx?wa=wsignin1.0'

    @property
    def tenant(self):
        return urlparse(self.authorityUrl).netloc

    @property
    def security_token_service_url(self):
        return "/".join([self.serviceUrl, self.securityTokenServicePath])

    @property
    def signin_page_url(self):
        return "/".join([self.authorityUrl, self.signInPage])

    @property
    def user_realm_service_url(self):
        return '/'.join([self.serviceUrl, self.userRealmServicePath])



from office365.runtime.client_value import ClientValue
from office365.runtime.odata.json_light_format import JsonLightFormat


class ClientValueCollection(ClientValue):

    def __init__(self, item_type, initial_values=None):
        """
        :type item_type: any
        :type initial_values: [] or None
        """
        super(ClientValueCollection, self).__init__()
        if initial_values is None:
            initial_values = []
        self._data = initial_values
        self._item_type = item_type

    def add(self, value):
        self._data.append(value)

    def __getitem__(self, index):
        """

        :type index: int
        :rtype: ClientValue
        """
        return self._data[index]

    def __iter__(self):
        for item in self._data:
            yield item

    def __len__(self):
        return len(self._data)

    def to_json(self, json_format=None):
        """
        :type json_format: office365.runtime.odata.odata_json_format.ODataJsonFormat or None
        """
        json_items = [item for item in self._data]
        for i, v in enumerate(json_items):
            if isinstance(v, ClientValue):
                json_items[i] = v.to_json(json_format)
        if isinstance(json_format, JsonLightFormat) and json_format.is_verbose:
            json = {json_format.collection_tag_name: json_items,
                    json_format.metadata_type_tag_name: {'type': self.entity_type_name}}

            return json
        return json_items

    def set_property(self, index, value, persist_changes=False):
        child_value = self._item_type()
        if isinstance(child_value, ClientValue):
            for k, v in value.items():
                child_value.set_property(k, v, False)
        else:
            child_value = value
        self.add(child_value)

    @property
    def entity_type_name(self):
        """
        Gets server type name
        """
        primitive_types = {
            bool: "Edm.Boolean",
            int: "Edm.Int32",
            str: "Edm.String",
        }

        item_type_name = None

        is_primitive = primitive_types.get(self._item_type, None) is not None
        if is_primitive:
            item_type_name = primitive_types[self._item_type]
        elif issubclass(self._item_type, ClientValue):
            item_type_name = self._item_type.entity_type_name.fget(self)

        if item_type_name is not None:
            return "Collection({0})".format(item_type_name)
        else:
            return None



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.sharepoint.base_entity import BaseEntity


class VideoItem(BaseEntity):

    def get_video_embed_code(self, width, height, autoplay=True, showInfo=True, makeResponsive=True):
        """

        :type width: int
        :type height: int
        :type autoplay: bool
        :type showInfo: bool
        :type makeResponsive: bool
        """
        return_type = ClientResult(self.context)
        params = {"width": width, "height": height, "autoplay": autoplay,
                  "showInfo": showInfo, "makeResponsive": makeResponsive}
        qry = ServiceOperationQuery(self, "GetVideoEmbedCode", params, None, None, return_type)
        self.context.add_query(qry)
        return return_type



from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.publishing.site_page_metadata import SitePageMetadata


class SitePageMetadataCollection(BaseEntityCollection):

    def __init__(self, context, resource_path=None):
        """Specifies a collection of site pages."""
        super(SitePageMetadataCollection, self).__init__(context, SitePageMetadata, resource_path)



from office365.runtime.client_object import ClientObject
from office365.runtime.client_result import ClientResult
from office365.runtime.http.http_method import HttpMethod
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.portal.group_creation_params import GroupCreationInformation
from office365.sharepoint.portal.group_site_info import GroupSiteInfo


class GroupSiteManager(ClientObject):
    def __init__(self, context):
        super(GroupSiteManager, self).__init__(context, ResourcePath("GroupSiteManager"), None)

    def create_group_ex(self, display_name, alias, is_public, optional_params=None):
        """
        Create a modern site

        :param str display_name:
        :param str alias:
        :param bool is_public:
        :param office365.sharepoint.portal.group_creation_params.GroupCreationParams or None optional_params:
        """
        payload = GroupCreationInformation(display_name, alias, is_public, optional_params)
        result = ClientResult(self.context, GroupSiteInfo())
        qry = ServiceOperationQuery(self, "CreateGroupEx", None, payload, None, result)
        self.context.add_query(qry)
        return result

    def delete(self, site_url):
        """
        Deletes a SharePoint Team site

        :type site_url: str
        """
        payload = {
            "siteUrl": site_url
        }
        qry = ServiceOperationQuery(self, "Delete", None, payload)
        self.context.add_query(qry)
        return self

    def get_status(self, group_id):
        """Get the status of a SharePoint site

        :type group_id: str
        """
        result = ClientResult(self.context, GroupSiteInfo())
        qry = ServiceOperationQuery(self, "GetSiteStatus", None, {'groupId': group_id}, None, result)
        self.context.add_query(qry)

        def _construct_status_request(request):
            request.method = HttpMethod.Get
            request.url += "?groupId='{0}'".format(group_id)

        self.context.before_execute(_construct_status_request)
        return result



from office365.runtime.client_value import ClientValue


class WebCreationInformation(ClientValue):
    """Represents metadata about site creation."""

    def __init__(self):
        super(WebCreationInformation, self).__init__()
        self.Title = None
        self.Url = None

    @property
    def entity_type_name(self):
        return "SP.WebCreationInformation"



from office365.runtime.client_value import ClientValue


class DriveRecipient(ClientValue):
    """
    The DriveRecipient resource represents a person, group, or other recipient to
    share with using the invite action.
    """

    def __init__(self):
        super(DriveRecipient, self).__init__()
        self.alias = None
        self.email = None
        self.objectId = None



from office365.sharepoint.fields.field import Field


class FieldMultiLineText(Field):

    def __init__(self, context):
        """Represents a text field that can contain multiple lines."""
        super(FieldMultiLineText, self).__init__(context)

    @property
    def allow_hyperlink(self):
        """
        Gets a value that specifies whether a hyperlink is allowed as a value of the field.
        :rtype: bool or None
        """
        return self.properties.get("AllowHyperlink", None)

    @allow_hyperlink.setter
    def allow_hyperlink(self, val):
        """
        Sets a value that specifies whether a hyperlink is allowed as a value of the field.
        """
        self.set_property("AllowHyperlink", val)

    @property
    def number_of_lines(self):
        """
        Gets the number of lines to display in the field.
        :return:
        """
        return self.properties.get("NumberOfLines", None)

    @number_of_lines.setter
    def number_of_lines(self, val):
        """
        Set the number of lines to display in the field.
        """
        self.set_property("NumberOfLines", val)



from office365.runtime.client_object_collection import ClientObjectCollection


class BaseEntityCollection(ClientObjectCollection):
    """Represents a collection of View resources."""

    def __init__(self, context, child_item_type=None, resource_path=None, parent=None):
        """
        SharePoint entity set

        :param office365.sharepoint.client_context.ClientContext context: SharePoint context
        :type child_item_type: type[ClientObject]
        :type resource_path: office365.runtime.resource_path.ResourcePath
        :type parent: office365.sharepoint.base_entity.BaseEntity or None
        """
        super(BaseEntityCollection, self).__init__(context, child_item_type, resource_path)
        self._parent = parent



from office365.directory.directory_object import DirectoryObject


class ExtensionProperty(DirectoryObject):
    """
    Represents a directory extension that can be used to add a custom property to directory objects without
    requiring an external data store. For example, if an organization has a line of business (LOB) application
    that requires a Skype ID for each user in the directory, Microsoft Graph can be used to register a new property
    named skypeId on the directory’s User object, and then write a value to the new property for a specific user.
    """
    pass



from office365.runtime.client_value import ClientValue
from office365.runtime.client_value_collection import ClientValueCollection
from office365.sharepoint.fields.field import Field
from office365.sharepoint.fields.field_type import FieldType


class FieldCreationInformation(ClientValue):

    def __init__(self, title, field_type_kind, description=None,
                 lookup_list_id=None, lookup_field_name=None, lookup_web_id=None,
                 required=False):
        """
        Represents metadata about fields creation.

        :type lookup_web_id: str
        :type required: bool
        :type lookup_field_name: str
        :type lookup_list_id: str
        :type title: str
        :type field_type_kind: int
        :type description: str or None
        """
        super(FieldCreationInformation, self).__init__()
        self.Title = title
        self.FieldTypeKind = field_type_kind
        self.Description = description
        self.Choices = ClientValueCollection(str) \
            if field_type_kind == FieldType.MultiChoice or field_type_kind == FieldType.Choice else None
        self.LookupListId = lookup_list_id
        self.LookupFieldName = lookup_field_name
        self.LookupWebId = lookup_web_id
        self.Required = required

    @property
    def entity_type_name(self):
        return "SP.FieldCreationInformation"



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.tenant.administration.site_properties import SiteProperties


class SitePropertiesCollection(BaseEntityCollection):
    """CSiteProperties resource collection"""
    def __init__(self, context, resource_path=None):
        super(SitePropertiesCollection, self).__init__(context, SiteProperties, resource_path)

    def get_by_id(self, site_id):
        site_props = SiteProperties(self.context)
        qry = ServiceOperationQuery(self, "GetById", [site_id], None, None, site_props)
        self.context.add_query(qry)
        return site_props

    def get_lock_state_by_id(self, site_id):
        result = ClientResult(self.context)
        qry = ServiceOperationQuery(self, "GetLockStateById", [site_id], None, None, result)
        self.context.add_query(qry)
        return result



from office365.runtime.client_value import ClientValue


class TeamMemberSettings(ClientValue):
    """Settings to configure whether members can perform certain actions, for example, create channels and add bots,
    in the team. """

    def __init__(self):
        super(TeamMemberSettings, self).__init__()
        self.allowCreateUpdateChannels = True
        self.allowDeleteChannels = True
        self.allowAddRemoveApps = True
        self.allowCreateUpdateRemoveTabs = True
        self.allowCreateUpdateRemoveConnectors = True



from office365.runtime.client_value import ClientValue


class NavigationNodeCreationInformation(ClientValue):

    def __init__(self, title=None, url=None, is_external=False, as_last_node=False, previous_node=None):
        """
        Describes a new navigation node to be created.

        :param NavigationNodeCreationInformation previous_node: Gets or sets a value that specifies the navigation node
            after which the new navigation node will appear in the navigation node collection.
        :param bool as_last_node: Gets or sets a value that specifies whether the navigation node will be created
            as the last node in the collection.
        :param str url: Gets or sets a value that specifies the URL to be stored with the navigation node.
        :param str title: Gets or sets a value that specifies the anchor text for the node navigation link.
        :param bool is_external: Gets or sets a value that specifies whether the navigation node URL potentially
            corresponds to pages outside of the site collection.


        """
        super(NavigationNodeCreationInformation, self).__init__()
        self.Title = title
        self.Url = url
        self.IsExternal = is_external
        self.AsLastNode = as_last_node
        self.PreviousNode = previous_node

    @property
    def entity_type_name(self):
        return "SP.NavigationNode"



from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.sharing.pickerSettings import PickerSettings


class SharePointSharingSettings(BaseEntity):

    @property
    def picker_properties(self):
        return self.properties.get('PickerProperties',
                                   PickerSettings(self.context, ResourcePath("PickerProperties", self.resource_path)))



from office365.sharepoint.changes.change import Change


class ChangeContentType(Change):
    """Specifies a change on a content type."""

    @property
    def content_type_id(self):
        return self.properties.get("ContentTypeId", None)



from office365.runtime.auth.client_credential import ClientCredential
from office365.runtime.auth.providers.acs_token_provider import ACSTokenProvider
from office365.runtime.auth.providers.oauth_token_provider import OAuthTokenProvider
from office365.runtime.auth.providers.saml_token_provider import SamlTokenProvider
from office365.runtime.auth.token_response import TokenResponse
from office365.runtime.auth.user_credential import UserCredential


class AuthenticationContext(object):

    def __init__(self, url):
        """
        Authentication context for SharePoint Online/OneDrive

        :param str url:  authority url
        """
        self.url = url
        self._provider = None

    def with_client_certificate(self, tenant, client_id, thumbprint, cert_path):
        """Creates authenticated SharePoint context via certificate credentials

        :param str tenant: Tenant name, for example {}@
        :param str cert_path: Path to A PEM encoded certificate private key.
        :param str thumbprint: Hex encoded thumbprint of the certificate.
        :param str client_id: The OAuth client id of the calling application.
        """

        def _acquire_token_for_client_certificate():
            authority_url = 'https://login.microsoftonline.com/{0}'.format(tenant)
            scopes = ["{url}/.default".format(url=self.url)]
            credentials = {"thumbprint": thumbprint, "private_key": open(cert_path).read()}
            import msal
            app = msal.ConfidentialClientApplication(
                client_id,
                authority=authority_url,
                client_credential=credentials,
            )
            result = app.acquire_token_for_client(scopes)
            return TokenResponse.from_json(result)

        self.register_provider(_acquire_token_for_client_certificate)
        return self

    def register_provider(self, credentials_or_token_func, **kwargs):
        if callable(credentials_or_token_func):
            self._provider = OAuthTokenProvider(credentials_or_token_func)
        elif isinstance(credentials_or_token_func, ClientCredential):
            self._provider = ACSTokenProvider(self.url, credentials_or_token_func.clientId,
                                              credentials_or_token_func.clientSecret)
        elif isinstance(credentials_or_token_func, UserCredential):
            allow_ntlm = kwargs.get('allow_ntlm', False)
            if allow_ntlm:
                from office365.runtime.auth.providers.ntlm_provider import NtlmProvider
                self._provider = NtlmProvider(credentials_or_token_func.userName,
                                              credentials_or_token_func.password)
            else:
                browser_mode = kwargs.get('browser_mode', False)
                self._provider = SamlTokenProvider(self.url, credentials_or_token_func.userName,
                                                   credentials_or_token_func.password, browser_mode)
        else:
            raise ValueError("Unknown credential type")

    def acquire_token_for_user(self, username, password, browser_mode=False):
        """Acquire token via user credentials
        Status: deprecated!

        :type password: str
        :type username: str
        :type browser_mode: str
        """
        self._provider = SamlTokenProvider(url=self.url, username=username, password=password,
                                           browser_mode=browser_mode)
        return self._provider.ensure_authentication_cookie()

    def acquire_token_for_app(self, client_id, client_secret):
        """Acquire token via client credentials (SharePoint App Principal)
        Status: deprecated!
        """
        self._provider = ACSTokenProvider(url=self.url, client_id=client_id, client_secret=client_secret)
        return self._provider.ensure_app_only_access_token()

    def authenticate_request(self, request):
        """Authenticate request
        :type request: RequestOptions"""
        self._provider.authenticate_request(request)



from office365.runtime.client_value import ClientValue


class ServicePlanInfo(ClientValue):
    """Contains information about a service plan associated with a subscribed SKU. The servicePlans property of
    the subscribedSku entity is a collection of servicePlanInfo."""

    def __init__(self, _id=None, name=None, provisioning_status=None, applies_to=None):
        """

        :param str applies_to: The object the service plan can be assigned to. Possible values:
               "User" - service plan can be assigned to individual users.
               "Company" - service plan can be assigned to the entire tenant.
        :param str provisioning_status: The provisioning status of the service plan. Possible values:
               "Success" - Service is fully provisioned.
               "Disabled" - Service has been disabled.
               "PendingInput" - Service is not yet provisioned; awaiting service confirmation.
               "PendingActivation" - Service is provisioned but requires explicit activation by administrator
               (for example, Intune_O365 service plan)
               "PendingProvisioning" - Microsoft has added a new service to the product SKU and it has not been
               activated in the tenant, yet.
        :param str name: The name of the service plan.
        :param str _id: The unique identifier of the service plan.
        """
        super(ServicePlanInfo, self).__init__()
        self.servicePlanId = _id
        self.servicePlanName = name
        self.provisioningStatus = provisioning_status
        self.appliesTo = applies_to



import pytz
from office365.runtime.client_value import ClientValue


class DateTimeTimeZone(ClientValue):
    """Describes the date, time, and time zone of a point in time."""

    def __init__(self, dateTime=None, timeZone=None):
        """

        :param str timeZone: Represents a time zone, for example, "Pacific Standard Time".
        :param str dateTime: A single point of time in a combined date and time representation ({date}T{time};
            for example, 2017-08-29T04:00:00.0000000).
        """
        super(DateTimeTimeZone, self).__init__()
        self.dateTime = dateTime
        self.timeZone = timeZone

    @staticmethod
    def parse(dt):
        """
        :type dt: datetime.datetime
        """
        local_dt = dt.replace(tzinfo=pytz.utc)
        return DateTimeTimeZone(dateTime=local_dt.isoformat(), timeZone=local_dt.strftime('%Z'))



from office365.sharepoint.fields.field import Field


class FieldDateTime(Field):
    """Specifies a field (2) that contains date and time values. To set properties, call the Update method
    (section 3.2.5.44.2.1.5)."""



from office365.runtime.client_value import ClientValue


class Reminder(ClientValue):
    """A reminder for an event in a user calendar."""

    def __init__(self, eventId=None):
        """

        :param str eventId: The unique ID of the event. Read only.
        """
        super(Reminder, self).__init__()
        self.eventId = eventId



from office365.runtime.client_value import ClientValue


class SPSiteCreationRequest(ClientValue):

    def __init__(self, title, url, owner=None):
        super(SPSiteCreationRequest, self).__init__()
        self.Title = title
        self.Url = url
        self.WebTemplate = "SITEPAGEPUBLISHING#0"
        self.Owner = owner
        self.Lcid = 1033
        self.ShareByEmailEnabled = False
        self.Classification = ""
        self.Description = ""
        self.SiteDesignId = "00000000-0000-0000-0000-000000000000"
        self.HubSiteId = "00000000-0000-0000-0000-000000000000"
        self.WebTemplateExtensionId = "00000000-0000-0000-0000-000000000000"

    @property
    def entity_type_name(self):
        return "Microsoft.SharePoint.Portal.SPSiteCreationRequest"



from office365.runtime.client_object import ClientObject
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.userprofiles.userProfile import UserProfile


class ProfileLoader(ClientObject):

    def __init__(self, context):
        super(ProfileLoader, self).__init__(context, ResourcePath("SP.UserProfiles.ProfileLoader.GetProfileLoader"))

    @staticmethod
    def get_profile_loader(context):
        """
        :type: office365.sharepoint.client_context.ClientContext context
        """
        result = ProfileLoader(context)
        qry = ServiceOperationQuery(result, "GetProfileLoader", None, None, None, result)
        qry.static = True
        context.add_query(qry)
        return result

    def get_user_profile(self):
        result = UserProfile(self.context, ResourcePath("GetUserProfile", self.resource_path))
        qry = ServiceOperationQuery(self, "GetUserProfile", None, None, None, result)
        self.context.add_query(qry)
        return result

    @property
    def entity_type_name(self):
        return "SP.UserProfiles.ProfileLoader"



from office365.runtime.client_value import ClientValue


class ChoiceColumn(ClientValue):
    """
    The choiceColumn on a columnDefinition resource indicates that the column's values can be selected
    from a list of choices.
    """
    pass



from office365.runtime.queries.delete_entity_query import DeleteEntityQuery
from office365.runtime.resource_path import ResourcePath
from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
from office365.sharepoint.base_entity import BaseEntity


class NavigationNode(BaseEntity):
    """
    Represents the URL to a specific navigation node and provides access to properties and methods for
    manipulating the ordering of the navigation node in a navigation node collection.
    """

    @property
    def children(self):
        """Gets the collection of child nodes of the navigation node."""
        from office365.sharepoint.navigation.navigation_node_collection import NavigationNodeCollection
        return self.properties.get('Children',
                                   NavigationNodeCollection(self.context, ResourcePath("Children", self.resource_path)))

    @property
    def title(self):
        """Gets a value that specifies the anchor text for the navigation node link."""
        return self.properties.get('Title', None)

    @title.setter
    def title(self, value):
        """Sets a value that specifies the anchor text for the navigation node link."""
        self.set_property('Title', value)

    @property
    def url(self):
        """Gets a value that specifies the URL stored with the navigation node."""
        return self.properties.get('Url', None)

    @url.setter
    def url(self, value):
        """Sets a value that specifies the URL stored with the navigation node."""
        self.set_property('Url', value)

    @property
    def is_visible(self):
        """Gets a value that specifies the anchor text for the navigation node link.
        :rtype: bool or None
        """
        return self.properties.get('isVisible', None)

    @property
    def is_external(self):
        """
        :rtype: bool or None
        """
        return self.properties.get('isExternal', None)

    def set_property(self, name, value, persist_changes=True):
        super(NavigationNode, self).set_property(name, value, persist_changes)
        # fallback: create a new resource path
        if self._resource_path is None:
            if name == "Id":
                self._resource_path = ResourcePathServiceOperation("GetById",
                                                                   [value],
                                                                   self._parent_collection.resource_path)
        return self



from office365.runtime.client_request import ClientRequest
from office365.runtime.http.http_method import HttpMethod
from office365.runtime.http.request_options import RequestOptions


class ODataV4BatchRequest(ClientRequest):
    """ JSON batch request """

    def __init__(self, context):
        super(ODataV4BatchRequest, self).__init__(context)

    def build_request(self):
        url = "{0}/$batch".format(self.context.service_root_url())
        request = RequestOptions(url)
        request.method = HttpMethod.Post
        request.ensure_header('Content-Type', "application/json")
        request.ensure_header('Accept', "application/json")
        request.data = self._prepare_payload()
        return request

    def process_response(self, response):
        """Parses an HTTP response.

        :type response: requests.Response
        """
        json = response.json()
        for resp in json["responses"]:
            self._validate_response(resp)
            sub_qry = self.current_query.get(int(resp["id"]))
            self.context.pending_request().map_json(resp["body"], sub_qry.return_type)

    def _validate_response(self, json):
        if int(json['status']) >= 400:
            raise ValueError(json['body'])

    def _prepare_payload(self):
        """
        Serializes a batch request body.
        """

        requests_json = []
        for qry in self.current_query.queries:
            request_id = str(len(requests_json))
            request = qry.build_request()
            requests_json.append(self._normalize_request(request, request_id))

        return {"requests": requests_json}

    def _normalize_request(self, request, _id, depends_on=None):
        """

        :type request: RequestOptions
        :type _id: str
        :type depends_on:  list[str] or None
        """
        allowed_props = ["id", "method", "headers", "url", "body"]
        json = dict((k, v) for k, v in vars(request).items() if v is not None and k in allowed_props)
        json["id"] = _id
        if depends_on is not None:
            json["dependsOn"] = depends_on
        json["url"] = json["url"].replace(self.context.service_root_url(), "")
        return json

    @property
    def current_query(self):
        """
        :rtype: office365.runtime.queries.batch_query.BatchQuery
        """
        return self._current_query



from office365.runtime.client_value import ClientValue


class AttachmentItem(ClientValue):
    """Represents attributes of an item to be attached."""

    def __init__(self, attachment_type, name, size):
        super(AttachmentItem, self).__init__()
        self.attachmentType = attachment_type
        self.name = name
        self.size = size



from office365.directory.extensions.extension import Extension
from office365.entity_collection import EntityCollection
from office365.outlook.mail.attachment_collection import AttachmentCollection
from office365.outlook.mail.item import Item
from office365.outlook.mail.itemBody import ItemBody
from office365.outlook.mail.recipient import Recipient
from office365.runtime.client_value_collection import ClientValueCollection
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath


class Message(Item):
    """A message in a mailbox folder."""

    def send(self):
        """
        Send a message in the draft folder. The draft message can be a new message draft, reply draft, reply-all draft,
        or a forward draft. The message is then saved in the Sent Items folder.
        """
        qry = ServiceOperationQuery(self, "send")
        self.context.add_query(qry)
        return self

    def reply(self):
        """Reply to the sender of a message by specifying a comment and using the Reply method. The message is then
        saved in the Sent Items folder. """
        qry = ServiceOperationQuery(self, "reply")
        self.context.add_query(qry)
        return self

    def reply_all(self):
        """Reply to all recipients of a message. The message is then saved in the Sent Items folder. """
        qry = ServiceOperationQuery(self, "replyAll")
        self.context.add_query(qry)
        return self

    def create_reply_all(self):
        """
        Create a draft to reply to the sender and all the recipients of the specified message.
        You can then update the draft to add reply content to the body or change other message properties, or,
        simply send the draft.
        :return:
        """
        qry = ServiceOperationQuery(self, "createReplyAll")
        self.context.add_query(qry)
        return self

    def move(self):
        """
        Move a message to another folder within the specified user's mailbox.
        This creates a new copy of the message in the destination folder and removes the original message.
        """
        qry = ServiceOperationQuery(self, "move")
        self.context.add_query(qry)
        return self

    def forward(self, to_recipients, comment=""):
        """
        Forward a message. The message is saved in the Sent Items folder.
        :param list[str] to_recipients: The list of recipients.
        :param str comment: A comment to include. Can be an empty string.
        """
        payload = {
            "toRecipients": ClientValueCollection(Recipient,
                                                  [Recipient.from_email(v) for v in to_recipients]),
            "comment": comment
        }
        qry = ServiceOperationQuery(self, "forward", None, payload)
        self.context.add_query(qry)
        return self

    @property
    def has_attachments(self):
        """
        Indicates whether the message has attachments. This property doesn't include inline attachments,
        so if a message contains only inline attachments, this property is false. To verify the existence
        of inline attachments, parse the body property to look for a src attribute,
        such as <IMG src="cid:image001.jpg@01D26CD8.6C05F070">.

        :rtype: bool or None
        """
        return self.properties.get("hasAttachments", None)

    @property
    def attachments(self):
        """The fileAttachment and itemAttachment attachments for the message."""
        return self.properties.get('attachments',
                                   AttachmentCollection(self.context, ResourcePath("attachments", self.resource_path)))

    @attachments.setter
    def attachments(self, value):
        """
        Sets the fileAttachment and itemAttachment attachments for the message.
        :type value list[office365.mail.attachment.Attachment]
        """
        col = AttachmentCollection(self.context, ResourcePath("attachments", self.resource_path))
        [col.add_child(item) for item in value]
        self.set_property('attachments', col)

    @property
    def extensions(self):
        """The collection of open extensions defined for the message. Nullable."""
        return self.properties.get('extensions',
                                   EntityCollection(self.context, Extension,
                                                    ResourcePath("extensions", self.resource_path)))

    @property
    def body(self):
        """The body of the message. It can be in HTML or text format."""
        return self.get_property("body", ItemBody())

    @body.setter
    def body(self, value):
        """The body of the message. It can be in HTML or text format.

        :type value: str or ItemBody
        """
        if not isinstance(value, ItemBody):
            value = ItemBody(value)
        self.set_property("body", value)

    @property
    def subject(self):
        """The subject of the message."""
        return self.properties.get("subject", None)

    @subject.setter
    def subject(self, value):
        """The subject of the message.

        :type value: str
        """
        self.set_property("subject", value)

    @property
    def to_recipients(self):
        """The To: recipients for the message."""
        return self.properties.get('toRecipients', ClientValueCollection(Recipient))

    @to_recipients.setter
    def to_recipients(self, value):
        """
        The To: recipients for the message.

        :type value: list[str] or list[Recipient]
        """
        self.set_property('toRecipients',
                          ClientValueCollection(Recipient, [Recipient.from_email(email) for email in value]))

    def get_property(self, name, default_value=None):
        if default_value is None:
            property_type_mapping = {
                "toRecipients": self.to_recipients
            }
            default_value = property_type_mapping.get(name, None)
        return super(Message, self).get_property(name, default_value)



from office365.base_item import BaseItem
from office365.entity_collection import EntityCollection
from office365.onedrive.columns.column_definition import ColumnDefinition
from office365.onedrive.contenttypes.content_type import ContentType
from office365.onedrive.drives.drive import Drive
from office365.onedrive.itemAnalytics import ItemAnalytics
from office365.onedrive.lists.list_collection import ListCollection
from office365.onedrive.listitems.listItem import ListItem
from office365.onedrive.permissions.permission import Permission
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath


class Site(BaseItem):
    """The site resource provides metadata and relationships for a SharePoint site. """

    def get_by_path(self, path):
        """
        Retrieve properties and relationships for a site resource. A site resource represents a team site in SharePoint.

            In addition to retrieving a site by ID you can retrieve a site based on server-relative URL path.

            Site collection hostname (contoso.sharepoint.com)
            Site path, relative to server hostname.
            There is also a reserved site identifier, root, which always references the root site for a given target,
            as follows:

            /sites/root: The tenant root site.
            /groups/{group-id}/sites/root: The group's team site.

        :type path: str
        """
        return_site = Site(self.context)
        qry = ServiceOperationQuery(self, "GetByPath", [path], None, None, return_site)
        self.context.add_query(qry)
        return return_site

    @property
    def items(self):
        """Used to address any item contained in this site. This collection cannot be enumerated."""
        return self.get_property('items',
                                 EntityCollection(self.context, ListItem, ResourcePath("items", self.resource_path)))

    @property
    def columns(self):
        """The collection of columns under this site."""
        return self.properties.get('columns',
                                   EntityCollection(self.context, ColumnDefinition,
                                                    ResourcePath("columns", self.resource_path)))

    @property
    def content_types(self):
        """The collection of content types under this site."""
        return self.properties.get('contentTypes',
                                   EntityCollection(self.context, ContentType,
                                                    ResourcePath("contentTypes", self.resource_path)))

    @property
    def lists(self):
        """The collection of lists under this site."""
        return self.properties.get('lists',
                                   ListCollection(self.context, ResourcePath("lists", self.resource_path)))

    @property
    def permissions(self):
        """The collection of lists under this site."""
        return self.properties.get('permissions',
                                   EntityCollection(self.context, Permission,
                                                    ResourcePath("permissions", self.resource_path)))

    @property
    def drive(self):
        """The default drive (document library) for this site."""
        return self.properties.get('drive',
                                   Drive(self.context, ResourcePath("drive", self.resource_path)))

    @property
    def drives(self):
        """The collection of drives under this site."""
        return self.properties.get('drives',
                                   EntityCollection(self.context, Drive, ResourcePath("drives", self.resource_path)))

    @property
    def sharepoint_ids(self):
        """Returns identifiers useful for SharePoint REST compatibility."""
        return self.properties.get('sharepointIds', None)

    @property
    def sites(self):
        """The collection of sites under this site."""
        from office365.onedrive.sites.site_collection import SiteCollection
        return self.properties.get('sites',
                                   SiteCollection(self.context, ResourcePath("sites", self.resource_path)))

    @property
    def analytics(self):
        """Analytics about the view activities that took place on this site."""
        return self.properties.get('analytics',
                                   ItemAnalytics(self.context, ResourcePath("analytics", self.resource_path)))

    def set_property(self, name, value, persist_changes=True):
        super(Site, self).set_property(name, value, persist_changes)
        if name == "id" and self._resource_path.segment == "root":
            self._resource_path = ResourcePath(value, self._resource_path.parent)
        return self



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.changes.change_collection import ChangeCollection
from office365.sharepoint.eventreceivers.event_receiver_definition import EventReceiverDefinitionCollection
from office365.sharepoint.features.feature_collection import FeatureCollection
from office365.sharepoint.lists.list import List
from office365.sharepoint.principal.user import User
from office365.sharepoint.recyclebin.recycleBinItemCollection import RecycleBinItemCollection
from office365.sharepoint.sites.sph_site import SPHSite
from office365.sharepoint.webs.web import Web
from office365.sharepoint.webs.web_template_collection import WebTemplateCollection
from office365.sharepoint.types.resource_path import ResourcePath as SPResPath


class Site(BaseEntity):
    """Represents a collection of sites in a Web application, including a top-level website and all its sub sites."""

    def __init__(self, context):
        super(Site, self).__init__(context, ResourcePath("Site", None))

    @staticmethod
    def from_url(url):
        from office365.sharepoint.client_context import ClientContext
        client = ClientContext(url)
        return client.site

    def is_valid_home_site(self):
        result = ClientResult(self.context)

        def _site_loaded():
            SPHSite.is_valid_home_site(self.context, self.url, result)

        self.ensure_property("Url", _site_loaded)
        return result

    def set_as_home_site(self):
        result = ClientResult(self.context)

        def _site_loaded():
            self.result = SPHSite.set_as_home_site(self.context, self.url, result)

        self.ensure_property("Url", _site_loaded)
        return result

    def get_changes(self, query):
        """Returns the collection of all changes from the change log that have occurred within the scope of the site,
        based on the specified query.

        :param office365.sharepoint.changes.change_query.ChangeQuery query: Specifies which changes to return
        """
        changes = ChangeCollection(self.context)
        qry = ServiceOperationQuery(self, "getChanges", None, query, "query", changes)
        self.context.add_query(qry)
        return changes

    def get_recycle_bin_items(self, rowLimit=100, isAscending=True):
        """

        :param int rowLimit:
        :param bool isAscending:
        """
        result = RecycleBinItemCollection(self.context)
        payload = {
            "rowLimit": rowLimit,
            "isAscending": isAscending
        }
        qry = ServiceOperationQuery(self, "GetRecycleBinItems", None, payload, None, result)
        self.context.add_query(qry)
        return result

    def get_web_templates(self, lcid=1033, override_compat_level=0):
        """
        Returns the collection of site definitions that are available for creating
            Web sites within the site collection.<99>

        :param int lcid: A 32-bit unsigned integer that specifies the language of the site definitions that are
            returned from the site collection.
        :param int override_compat_level: Specifies the compatibility level of the site (2)
            to return from the site collection. If this value is 0, the compatibility level of the site (2) is used.
        :return:
        """
        params = {
            "LCID": lcid,
            "overrideCompatLevel": override_compat_level
        }
        return_type = WebTemplateCollection(self.context,
                                            ResourcePathServiceOperation("GetWebTemplates", params, self.resource_path))

        qry = ServiceOperationQuery(self, "GetWebTemplates", params, None, None, return_type)
        self.context.add_query(qry)
        return return_type

    def join_hub_site(self, hubSiteId, approvalToken, approvalCorrelationId):
        params = {
            "hubSiteId": hubSiteId,
            "approvalToken": approvalToken,
            "approvalCorrelationId": approvalCorrelationId
        }
        return_type = WebTemplateCollection(self.context,
                                            ResourcePathServiceOperation("GetWebTemplates", params, self.resource_path))

        qry = ServiceOperationQuery(self, "JoinHubSite", params, None, None, return_type)
        self.context.add_query(qry)
        return return_type

    @staticmethod
    def get_url_by_id(context, site_id, stop_redirect=False):
        """Gets Site Url By Id

        :type context: office365.sharepoint.client_context.ClientContext
        :type site_id: str
        :type stop_redirect: bool
        """
        result = ClientResult(context)
        payload = {
            "id": site_id,
            "stopRedirect": stop_redirect
        }
        qry = ServiceOperationQuery(context.site, "GetUrlById", None, payload, None, result)
        qry.static = True
        context.add_query(qry)
        return result

    @staticmethod
    def get_url_by_id_for_web(context):
        pass

    @staticmethod
    def exists(context, url):
        """Determine whether site exists
        :type context: office365.sharepoint.client_context.ClientContext
        :type url: str
        """
        result = ClientResult(context)
        payload = {
            "url": url
        }
        qry = ServiceOperationQuery(context.site, "Exists", None, payload, None, result)
        qry.static = True
        context.add_query(qry)
        return result

    def get_catalog(self, type_catalog):
        """Specifies the list template gallery, site template gallery, Web Part gallery, master page gallery,
        or other galleries from the site collection, including custom galleries that are defined by users.
        :type type_catalog: int"""
        return List(self.context, ResourcePathServiceOperation("getCatalog", [type_catalog], self.resource_path))

    def register_hub_site(self, creationInformation):
        """Registers an existing site as a hub site.

        :type creationInformation: HubSiteCreationInformation
        """
        qry = ServiceOperationQuery(self, "RegisterHubSite", None, creationInformation, "creationInformation", None)
        self.context.add_query(qry)
        return self

    def unregister_hub_site(self):
        qry = ServiceOperationQuery(self, "UnRegisterHubSite", None, None, None, None)
        self.context.add_query(qry)
        return self

    @property
    def root_web(self):
        """Get root web"""
        return self.properties.get('RootWeb', Web(self.context, ResourcePath("RootWeb", self.resource_path)))

    @property
    def owner(self):
        """Gets or sets the owner of the site collection. (Read-only in sandboxed solutions.)"""
        return self.properties.get('owner', User(self.context, ResourcePath("owner", self.resource_path)))

    @property
    def url(self):
        return self.properties.get('Url', None)

    @property
    def id(self):
        """
        :rtype: str
        """
        return self.properties.get("Id", None)

    @property
    def is_hub_site(self):
        """
        :rtype: bool
        """
        return self.properties.get("IsHubSite", None)

    @property
    def server_relative_path(self):
        """Gets the server-relative Path of the Site.
        :rtype: SPResPath or None
        """
        return self.properties.get("ServerRelativePath", SPResPath(None))

    @property
    def recycle_bin(self):
        """Get recycle bin"""
        return self.properties.get('RecycleBin',
                                   RecycleBinItemCollection(self.context,
                                                            ResourcePath("RecycleBin", self.resource_path)))

    @property
    def features(self):
        """Get features"""
        return self.properties.get('Features',
                                   FeatureCollection(self.context,
                                                     ResourcePath("Features", self.resource_path), self))

    @property
    def event_receivers(self):
        """Get Event receivers"""
        return self.properties.get('EventReceivers',
                                   EventReceiverDefinitionCollection(self.context,
                                                                     ResourcePath("eventReceivers", self.resource_path),
                                                                     self))

    def get_property(self, name, default_value=None):
        if name == "RecycleBin":
            default_value = self.recycle_bin
        elif name == "RootWeb":
            default_value = self.root_web
        elif name == "EventReceivers":
            default_value = self.event_receivers
        return super(Site, self).get_property(name, default_value)



from office365.runtime.client_result import ClientResult
from office365.runtime.http.http_method import HttpMethod
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.portal.site_creation_response import SPSiteCreationResponse


class SPSiteManager(BaseEntity):

    def __init__(self, context):
        super(SPSiteManager, self).__init__(context, ResourcePath("SPSiteManager"))

    def create(self, request):
        """Create a modern site"""
        result = ClientResult(self.context, SPSiteCreationResponse())
        qry = ServiceOperationQuery(self, "Create", None, request, "request", result)
        self.context.add_query(qry)
        return result

    def delete(self, site_id):
        """Deletes a SharePoint site"""
        payload = {
            "siteId": site_id
        }
        qry = ServiceOperationQuery(self, "Delete", None, payload)
        self.context.add_query(qry)
        return self

    def get_status(self, site_url):
        """Get the status of a SharePoint site

        :type site_url: str
        """
        response = ClientResult(self.context, SPSiteCreationResponse())
        qry = ServiceOperationQuery(self, "Status", None, {'url': site_url}, None, response)
        self.context.add_query(qry)

        def _construct_status_request(request):
            request.method = HttpMethod.Get
            request.url += "?url='{0}'".format(site_url)

        self.context.before_execute(_construct_status_request)
        return response



from office365.runtime.client_value import ClientValue
from office365.runtime.client_value_collection import ClientValueCollection


class GroupCreationInformation(ClientValue):

    def __init__(self, display_name, alias, is_public, optional_params=None):
        super(GroupCreationInformation, self).__init__()
        if optional_params is None:
            optional_params = GroupCreationParams()
        self.displayName = display_name
        self.alias = alias
        self.isPublic = is_public
        self.optionalParams = optional_params

    @property
    def entity_type_name(self):
        return None


class GroupCreationParams(ClientValue):

    def __init__(self, classification="", description=""):
        super(GroupCreationParams, self).__init__()
        self.Classification = classification
        self.Description = description
        self.CreationOptions = ClientValueCollection(str)
        self.CreationOptions.add("SPSiteLanguage:1033")

    @property
    def entity_type_name(self):
        return "Microsoft.SharePoint.Portal.GroupCreationParams"



class ViewScope:
    """Specifies the scope for returning list items and list folders in a list view."""
    def __init__(self):
        pass

    DefaultValue = "DefaultValue"
    Recursive = "Recursive"
    RecursiveAll = "RecursiveAll"
    FilesOnly = "FilesOnly"



from office365.entity_collection import EntityCollection
from office365.onedrive.internal.root_resource_path import RootResourcePath
from office365.onedrive.sites.site import Site


class SiteCollection(EntityCollection):
    """Drive site's collection"""

    def __init__(self, context, resource_path=None):
        super(SiteCollection, self).__init__(context, Site, resource_path)

    @property
    def root(self):
        """If present, indicates that this is a root site collection in SharePoint."""
        root_site = self.properties.get('root',
                                        Site(self.context, RootResourcePath(self.resource_path)))
        return root_site



from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
from office365.sharepoint.base_entity import BaseEntity


class FileVersion(BaseEntity):
    """Represents a version of a File object."""

    @property
    def url(self):
        """Gets a value that specifies the relative URL of the file version based on the URL for the site or subsite."""
        return self.properties.get("Url", None)

    @property
    def version_label(self):
        """Gets a value that specifies the implementation specific identifier of the file."""
        return self.properties.get("VersionLabel", None)

    @property
    def is_current_version(self):
        """Gets a value that specifies whether the file version is the current version."""
        return self.properties.get("IsCurrentVersion", None)

    @property
    def checkin_comment(self):
        """Gets a value that specifies the check-in comment."""
        return self.properties.get("CheckInComment", None)

    def set_property(self, name, value, persist_changes=True):
        super(FileVersion, self).set_property(name, value, persist_changes)
        if self._resource_path is None:
            if name == "ID":
                self._resource_path = ResourcePathServiceOperation(
                    "GetById",
                    [value],
                    self._parent_collection.resource_path)



from office365.runtime.client_value import ClientValue
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.sites.site_types import SiteCollectionAppCatalogAllowedItems


class TenantWebTemplate(ClientValue):

    def __init__(self):
        super(TenantWebTemplate, self).__init__()

    @property
    def entity_type_name(self):
        return "Microsoft.Online.SharePoint.TenantAdministration.SPOTenantWebTemplate"


class TenantAdminEndpoints(BaseEntity):
    pass


class TenantCorporateCatalogAccessor(BaseEntity):

    @property
    def site_collection_app_catalogs_sites(self):
        """Get recycle bin"""
        return self.properties.get('SiteCollectionAppCatalogsSites',
                                   SiteCollectionAppCatalogAllowedItems(self.context,
                                                                        ResourcePath("SiteCollectionAppCatalogsSites",
                                                                                     self.resource_path)))



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.fields.field_type import FieldType


class Field(BaseEntity):
    """A container for metadata within a SharePoint list and associated list items."""

    def __init__(self, context, resource_path=None):
        super(Field, self).__init__(context, resource_path)

    @staticmethod
    def resolve_field_type(type_id_or_name):
        from office365.sharepoint.fields.field_calculated import FieldCalculated
        from office365.sharepoint.fields.field_choice import FieldChoice
        from office365.sharepoint.fields.field_computed import FieldComputed
        from office365.sharepoint.fields.field_currency import FieldCurrency
        from office365.sharepoint.fields.field_guid import FieldGuid
        from office365.sharepoint.fields.field_lookup import FieldLookup
        from office365.sharepoint.fields.field_multi_choice import FieldMultiChoice
        from office365.sharepoint.fields.field_multi_line_text import FieldMultiLineText
        from office365.sharepoint.fields.field_text import FieldText
        from office365.sharepoint.fields.field_url import FieldUrl
        from office365.sharepoint.fields.field_user import FieldUser
        from office365.sharepoint.taxonomy.taxonomy_field import TaxonomyField
        from office365.sharepoint.fields.field_date_time import FieldDateTime
        field_known_types = {
            FieldType.Text: FieldText,
            FieldType.Calculated: FieldCalculated,
            FieldType.Choice: FieldChoice,
            FieldType.MultiChoice: FieldMultiChoice,
            FieldType.Lookup: FieldLookup,
            FieldType.User: FieldUser,
            FieldType.Computed: FieldComputed,
            FieldType.URL: FieldUrl,
            FieldType.Guid: FieldGuid,
            FieldType.Currency: FieldCurrency,
            FieldType.Note: FieldMultiLineText,
            FieldType.DateTime: FieldDateTime
        }
        if isinstance(type_id_or_name, int):
            return field_known_types.get(type_id_or_name, Field)
        else:
            if type_id_or_name == "TaxonomyFieldType" or type_id_or_name == "TaxonomyFieldTypeMulti":
                return TaxonomyField
            else:
                return Field

    @staticmethod
    def create_field_from_type(context, field_parameters):
        """

        :type context: ClientContext
        :type field_parameters: office365.sharepoint.fields.field_creation_information.FieldCreationInformation
        :return: Field
        """
        field_type = Field.resolve_field_type(field_parameters.FieldTypeKind)
        field = field_type(context)
        for n, v in field_parameters.to_json().items():
            field.set_property(n, v)
        return field

    def enable_index(self):
        result = ClientResult(self.context)
        qry = ServiceOperationQuery(self, "enableIndex", None, None, None, result)
        self.context.add_query(qry)
        return result

    def disable_index(self):
        result = ClientResult(self.context)
        qry = ServiceOperationQuery(self, "disableIndex", None, None, None, result)
        self.context.add_query(qry)
        return result

    def set_show_in_display_form(self, flag):
        """Sets the value of the ShowInDisplayForm property for this fields.

        :type flag: bool
        """
        qry = ServiceOperationQuery(self, "setShowInDisplayForm", [flag])
        self.context.add_query(qry)
        return self

    def set_show_in_edit_form(self, flag):
        """Sets the value of the ShowInEditForm property for this fields.
        :type flag: bool
        """
        qry = ServiceOperationQuery(self, "setShowInEditForm", [flag])
        self.context.add_query(qry)

    def set_show_in_new_form(self, flag):
        """Sets the value of the ShowInNewForm property for this fields."""
        qry = ServiceOperationQuery(self, "setShowInNewForm", [flag])
        self.context.add_query(qry)
        return self

    @property
    def id(self):
        """Gets a value that specifies the field identifier.

        :rtype: str or None
        """
        return self.properties.get('Id', None)

    @property
    def default_formula(self):
        """
        :rtype: str or None
        """
        return self.properties.get('DefaultFormula', None)

    @property
    def schema_xml(self):
        """Gets a value that specifies the XML schema that defines the field.

        :rtype: str or None
        """
        return self.properties.get('SchemaXml', None)

    @schema_xml.setter
    def schema_xml(self, val):
        """Sets a value that specifies the XML schema that defines the field.

        """
        self.set_property('SchemaXml', val)

    @property
    def type_as_string(self):
        """Gets a value that specifies the type of the field..

        :rtype: str or None
        """
        return self.properties.get('TypeAsString', None)

    @property
    def title(self):
        """Gets a value that specifies the display name of the field.

        :rtype: str or None
        """
        return self.properties.get('Title', None)

    @title.setter
    def title(self, val):
        """Sets a value that specifies the display name of the field.

        :rtype: str or None
        """
        self.set_property("Title", val)

    @property
    def group(self):
        """Gets a value that specifies the field group.

        :rtype: str or None
        """
        return self.properties.get('Group', None)

    @group.setter
    def group(self, val):
        """Sets a value that specifies the field group.

        :rtype: str or None
        """
        self.set_property("Group", val)

    @property
    def internal_name(self):
        """Gets a value that specifies the field internal name.

        :rtype: str or None
        """
        return self.properties.get('InternalName', None)

    @property
    def can_be_deleted(self):
        """Gets a value that specifies whether the field can be deleted

        :rtype: bool or None
        """
        return self.properties.get('CanBeDeleted', None)

    @property
    def client_side_component_id(self):
        """
        :rtype: str or None
        """
        return self.properties.get('ClientSideComponentId', None)

    @property
    def client_side_component_properties(self):
        """
        :rtype: str or None
        """
        return self.properties.get('ClientSideComponentProperties', None)

    @property
    def js_link(self):
        """
        When implemented in a derived class, gets or sets the name of an external JavaScript file that contains
        any client rendering logic for fields of the derived type.

        :rtype: str or None
        """
        return self.properties.get('JSLink', None)

    @property
    def hidden(self):
        """
        Gets a value that specifies whether the field is hidden in list views and list forms.

        :rtype: bool or None
        """
        return self.properties.get('Hidden', None)

    @hidden.setter
    def hidden(self, val):
        """
        Sets a value that specifies whether the field is hidden in list views and list forms.
        """
        self.set_property("Hidden", val)

    @property
    def default_value(self):
        """
        Gets  a value that specifies the default value for the field.

        :rtype: str or None
        """
        return self.properties.get('DefaultValue', None)

    @default_value.setter
    def default_value(self, val):
        """
        Sets a value that specifies the default value for the field.
        """
        self.set_property("DefaultValue", val)

    @property
    def type_display_name(self):
        """
        Gets a value that specifies the display name for the type of the field.

        :rtype: str or None
        """
        return self.properties.get('TypeDisplayName', None)

    @property
    def type_short_description(self):
        """
        Gets a value that specifies the description for the type of the field.

        :rtype: str or None
        """
        return self.properties.get('TypeShortDescription', None)

    def set_property(self, name, value, persist_changes=True):
        super(Field, self).set_property(name, value, persist_changes)
        # fallback: create a new resource path
        if name == "Id" and self._resource_path is None:
            self._resource_path = ResourcePathServiceOperation(
                "getById", [value], self._parent_collection.resource_path)
        if name == "FieldTypeKind":
            self.__class__ = self.resolve_field_type(value)
        elif name == "TypeAsString" and self.properties.get('FieldTypeKind', 0) == 0:
            self.__class__ = self.resolve_field_type(value)
        return self



from office365.sharepoint.base_entity import BaseEntity


class SocialFollowingManager(BaseEntity):
    """Provides methods for managing a user's list of followed actors (users, documents, sites, and tags)."""



from office365.runtime.client_value import ClientValue


class TeamFunSettings(ClientValue):
    """Settings to configure use of Giphy, memes, and stickers in the team."""

    def __init__(self):
        super(TeamFunSettings, self).__init__()
        self.allowGiphy = True



from office365.sharepoint.base_entity import BaseEntity


class InformationRightsManagementSettings(BaseEntity):

    @property
    def policy_title(self):
        """
        :rtype: str or None
        """
        return self.properties.get("PolicyTitle", None)

    @property
    def policy_description(self):
        """
        :rtype: str or None
        """
        return self.properties.get("PolicyDescription", None)

    @property
    def group_name(self):
        """
        :rtype: str or None
        """
        return self.properties.get("GroupName", None)



from office365.entity import Entity


class ProfilePhoto(Entity):
    """
    A profile photo of a user, group or an Outlook contact accessed from Exchange Online.
    It's binary data not encoded in base-64.

    The supported sizes of HD photos on Exchange Online are as follows: '48x48', '64x64', '96x96', '120x120',
    '240x240', '360x360','432x432', '504x504', and '648x648'.
    """

    @property
    def height(self):
        """
        The height of the photo. Read-only.
        """
        return self.properties.get('height', None)

    @property
    def width(self):
        """
        The width of the photo. Read-only.
        """
        return self.properties.get('width', None)



from office365.runtime.client_object import ClientObject
from office365.runtime.client_value import ClientValue


class PickerEntityInformation(ClientObject):
    pass


class PickerEntityInformationRequest(ClientValue):

    def __init__(self, EmailAddress=None, GroupId=None, Key=None, PrincipalType=None):
        super(PickerEntityInformationRequest, self).__init__()
        self.EmailAddress = EmailAddress
        self.GroupId = GroupId
        self.Key = Key
        self.PrincipalType = PrincipalType

    @property
    def entity_type_name(self):
        return "SP.UI.ApplicationPages.PickerEntityInformationRequest"



from office365.runtime.client_value_collection import ClientValueCollection
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.principal.group_collection import GroupCollection
from office365.sharepoint.sharing.invitationCreationResult import SPInvitationCreationResult
from office365.sharepoint.sharing.userSharingResult import UserSharingResult


class SharingResult(BaseEntity):

    @property
    def error_message(self):
        return self.properties.get("ErrorMessage", None)

    @property
    def name(self):
        return self.properties.get("Name", None)

    @property
    def icon_url(self):
        return self.properties.get("IconUrl", None)

    @property
    def statusCode(self):
        return self.properties.get("StatusCode", None)

    @property
    def permissions_page_relative_url(self):
        return self.properties.get("PermissionsPageRelativeUrl", None)

    @property
    def invited_users(self):
        return self.properties.get("InvitedUsers", ClientValueCollection(SPInvitationCreationResult))

    @property
    def uniquely_permissioned_users(self):
        return self.properties.get("UniquelyPermissionedUsers", ClientValueCollection(UserSharingResult))

    @property
    def groups_shared_with(self):
        return self.properties.get("GroupsSharedWith",
                                   GroupCollection(self.context, ResourcePath("GroupsSharedWith", self.resource_path)))



from office365.directory.directory_object import DirectoryObject


class ServicePrincipal(DirectoryObject):
    """Represents an instance of an application in a directory."""

    @property
    def app_display_name(self):
        """The collection of key credentials associated with the application. Not nullable.
        """
        return self.properties.get('appDisplayName', None)

    def add_key(self, key_credential, password_credential, proof):
        pass

    def add_password(self):
        pass



from requests import RequestException


class ClientRequestException(RequestException):
    def __init__(self, *args, **kwargs):
        super(ClientRequestException, self).__init__(*args, **kwargs)
        content_type = self.response.headers.get('Content-Type', '').lower().split(';')[0]
        if self.response.content and content_type == 'application/json':
            self.payload = self.response.json()
        else:
            self.payload = None
        args = (self.code, self.message) + args
        self.args = args

    @property
    def code(self):
        if self.payload:
            error = self.payload.get('error')
            if error:
                return error.get('code')

    @property
    def message_lang(self):
        if self.payload:
            error = self.payload.get('error')
            if error:
                message = error.get('message')
                if isinstance(message, dict):
                    return message.get('lang')

    @property
    def message(self):
        if self.payload:
            error = self.payload.get('error')
            if error:
                message = error.get('message')
                if isinstance(message, dict):
                    return message.get('value')
                return message



from office365.entity import Entity


class Attachment(Entity):
    """A file or item (contact, event or message) attached to an event or message."""

    @property
    def name(self):
        """
        The attachment's file name.
        :rtype: str or None
        """
        return self.properties.get("name", None)

    @name.setter
    def name(self, value):
        """
        Sets the attachment's file name.
        :type: value: str
        """
        self.set_property("name", value)

    @property
    def content_type(self):
        """
        :rtype: str or None
        """
        return self.properties.get("contentType", None)

    @content_type.setter
    def content_type(self, value):
        """
        :type: value: str
        """
        self.set_property("contentType", value)

    @property
    def size(self):
        """

        :rtype: int or None
        """
        return self.properties.get("size", None)

    @property
    def last_modified_date_time(self):
        """
        The Timestamp type represents date and time information using ISO 8601 format and is always in UTC time.

        :rtype: int or None
        """
        return self.properties.get("lastModifiedDateTime", None)




from office365.directory.audit.signIn_status import SignInStatus
from office365.entity import Entity


class SignIn(Entity):
    """Details user and application sign-in activity for a tenant (directory).
    You must have an Azure AD Premium P1 or P2 license to download sign-in logs using the Microsoft Graph API.
    """

    @property
    def user_id(self):
        """
        ID of the user that initiated the sign-in. Supports $filter (eq operator only).
        :rtype: str or None
        """
        return self.properties.get("userId", None)

    @property
    def user_principal_name(self):
        """
        User principal name of the user that initiated the sign-in. Supports $filter (eq and startsWith operators only).
        :rtype: str or None
        """
        return self.properties.get("userPrincipalName", None)

    @property
    def status(self):
        """
        Sign-in status. Includes the error code and description of the error (in case of a sign-in failure).
        Supports $filter (eq operator only) on errorCode property.
        :rtype: str or None
        """
        return self.properties.get("status", SignInStatus())



from office365.runtime.client_value import ClientValue


class ItemBody(ClientValue):
    """Represents properties of the body of an item, such as a message, event or group post."""

    def __init__(self, content=None, content_type="Text"):
        """

        :param str content: The content of the item.
        :param str content_type: The type of the content. Possible values are text and html.
        """
        super(ItemBody, self).__init__()
        self.content = content
        self.contentType = content_type



class ODataModel(object):
    """OData model"""
    _types = {}

    @property
    def types(self):
        return self._types

    def add_type(self, type_schema):
        """
        :type type_schema: office365.runtime.odata.odata_type.ODataType
        :return:
        """
        type_alias = type_schema.name
        self._types[type_alias] = type_schema



from office365.runtime.client_object import ClientObject
from office365.runtime.resource_path import ResourcePath


class SiteCollectionManagementService(ClientObject):

    def __init__(self, context):
        fqn = "Microsoft.Online.SharePoint.TenantAdministration.SiteCollectionManagementService"
        super(SiteCollectionManagementService, self).__init__(context, ResourcePath(fqn))



from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.sites.siteDesignPrincipal import SiteDesignPrincipalCollection


class SiteScriptUtility(BaseEntity):

    def __init__(self, context):
        super(SiteScriptUtility, self).__init__(context,
                                                ResourcePath(
                                                    "Microsoft.SharePoint.Utilities.WebTemplateExtensions.SiteScriptUtility"))

    @staticmethod
    def get_site_design_rights(context, _id):
        """

        :type _id: str
        :param office365.sharepoint.client_context.ClientContext context: client context

        """
        return_type = SiteDesignPrincipalCollection(context)
        utility = SiteScriptUtility(context)
        qry = ServiceOperationQuery(utility, "GetSiteDesignRights", [_id], None, None, return_type)
        qry.static = True
        context.add_query(qry)
        return return_type

    @staticmethod
    def get_list_design(context):
        pass



from office365.entity import Entity
from office365.entity_collection import EntityCollection


class WorkbookChart(Entity):
    pass


class WorkbookChartCollection(EntityCollection):

    def __init__(self, context, resource_path=None):
        super(WorkbookChartCollection, self).__init__(context, WorkbookChart, resource_path)


class WorkbookChartAxes(Entity):
    pass


class WorkbookChartDataLabels(Entity):
    pass


class WorkbookChartAreaFormat(Entity):
    pass


class WorkbookChartLegend(Entity):
    pass


class WorkbookChartSeries(Entity):
    pass



from office365.entity import Entity
from office365.entity_collection import EntityCollection
from office365.outlook.mail.message import Message
from office365.outlook.mail.message_rule import MessageRule
from office365.runtime.resource_path import ResourcePath


class MailFolder(Entity):
    """A mail folder in a user's mailbox, such as Inbox and Drafts. Mail folders can contain messages,
    other Outlook items, and child mail folders."""

    @property
    def total_item_count(self):
        """The number of items in the mailFolder."""
        return self.properties.get("totalItemCount", None)

    @property
    def child_folders(self):
        """The collection of child folders in the mailFolder. """
        return self.properties.get('childFolders',
                                   EntityCollection(self.context, MailFolder,
                                                    ResourcePath("childFolders", self.resource_path)))

    @property
    def message_rules(self):
        return self.properties.get('messageRules',
                                   EntityCollection(self.context, MessageRule,
                                                    ResourcePath("messageRules", self.resource_path)))

    @property
    def messages(self):
        """The collection of messages in the mailFolder."""
        return self.properties.get('messages',
                                   EntityCollection(self.context, Message,
                                                    ResourcePath("messages", self.resource_path)))



from office365.sharepoint.base_entity import BaseEntity


class SiteProperties(BaseEntity):
    """Contains a property bag of information about a site."""

    def __init__(self, context):
        super(SiteProperties, self).__init__(context)

    def set_property(self, name, value, persist_changes=True):
        super(SiteProperties, self).set_property(name, value, persist_changes)
        # fallback: create a new resource path
        if name == "Url" and self._resource_path is None:
            pass
            # site_ctx = self.context.clone(value)
            # target_site = site_ctx.site
            # site_ctx.load(target_site)
            # site_ctx.execute_query()
            # self._resource_path = ResourcePathServiceOperation(
            #    "getById", [target_site.properties['Id']], self._parent_collection.resource_path)

    @property
    def url(self):
        """

        :rtype: str
        """
        return self.properties.get('Url', None)

    @property
    def compatibilityLevel(self):
        """
        Gets the compatibility level of the site.

        :rtype: str
        """
        return self.properties.get('CompatibilityLevel', None)

    @property
    def lock_state(self):
        """
        Gets or sets the lock state of the site.

        :rtype: str
        """
        return self.properties.get('LockState', None)

    @property
    def entity_type_name(self):
        return "Microsoft.Online.SharePoint.TenantAdministration.SiteProperties"



from office365.runtime.resource_path import ResourcePath


class ChildrenResourcePath(ResourcePath):
    """Resource path for OneDrive children addressing"""

    def __init__(self, parent):
        super(ChildrenResourcePath, self).__init__("children", parent)



from office365.directory.audit.directory_audit import DirectoryAudit
from office365.directory.audit.provisioningObjectSummary import ProvisioningObjectSummary
from office365.directory.audit.signIn import SignIn
from office365.entity import Entity
from office365.entity_collection import EntityCollection
from office365.runtime.resource_path import ResourcePath


class AuditLogRoot(Entity):
    """Contains different types of audit logs. This resources returns a singleton auditLog resource.
    It doesn't contain any usable properties.
    """

    @property
    def directory_audits(self):
        """
        Get the list of audit logs generated by Azure Active Directory. This includes audit logs generated
        by various services within Azure AD, including user, app, device and group Management,
        privileged identity management (PIM), access reviews, terms of use, identity protection,
        password management (self-service and admin password resets), and self- service group management, and so on.

        :rtype: EntityCollection
        """
        return self.get_property('directoryAudits',
                                 EntityCollection(self.context, DirectoryAudit,
                                                  ResourcePath("directoryAudits", self.resource_path)))

    @property
    def signins(self):
        """
        Retrieve the Azure AD user sign-ins for your tenant. Sign-ins that are interactive in nature
        (where a username/password is passed as part of auth token) and successful federated sign-ins are currently
        included in the sign-in logs. The maximum and default page size is 1,000 objects and by default,
        the most recent sign-ins are returned first.

        :rtype: EntityCollection
        """
        return self.get_property('signIns',
                                 EntityCollection(self.context, SignIn, ResourcePath("signIns", self.resource_path)))

    @property
    def provisioning(self):
        """
        Get all provisioning events that occurred in your tenant, such as the deletion of a group in
        a target application or the creation of a user when provisioning user accounts from your HR system.

        :rtype: EntityCollection
        """
        return self.get_property('provisioning',
                                 EntityCollection(self.context, ProvisioningObjectSummary,
                                                  ResourcePath("provisioning", self.resource_path)))

    def get_property(self, name, default_value=None):
        if default_value is None:
            property_mapping = {
                "directoryAudits": self.directory_audits
            }
            default_value = property_mapping.get(name, None)
        return super(AuditLogRoot, self).get_property(name, default_value)



from generator.templates.entity_type import EntityType


class ProvisioningObjectSummary(EntityType):
    """Represents an action performed by the Azure AD Provisioning service and its associated properties."""
    pass



from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.lists.list_template import ListTemplate


class ListTemplateCollection(BaseEntityCollection):

    def __init__(self, context, resource_path=None):
        super(ListTemplateCollection, self).__init__(context, ListTemplate, resource_path)

    def get_by_name(self, name):
        """

        :param str name:
        :return:
        """
        return ListTemplate(self.context,
                            ResourcePathServiceOperation("getByName", [name], self.resource_path))



from office365.runtime.client_value import ClientValue


class SiteCreationProperties(ClientValue):

    def __init__(self, url, owner):
        """Sets the initial properties for a new site when it is created.
        :type owner: str
        :type url: str
        """
        super(SiteCreationProperties, self).__init__()
        self.Url = url
        self.Owner = owner
        self.Title = None
        self.Template = None

    @property
    def entity_type_name(self):
        return "Microsoft.Online.SharePoint.TenantAdministration.SiteCreationProperties"



from office365.runtime.odata.odata_json_format import ODataJsonFormat
from office365.runtime.odata.odata_metadata_level import ODataMetadataLevel


class JsonLightFormat(ODataJsonFormat):
    """JSON Light format for SharePoint Online/One Drive for Business"""

    def __init__(self, metadata=ODataMetadataLevel.Verbose):
        super(JsonLightFormat, self).__init__(metadata)
        if self.metadata == ODataMetadataLevel.Verbose:
            self.security_tag_name = "d"
            self.collection_tag_name = "results"
            self.collection_next_tag_name = "__next"
            self.metadata_type_tag_name = "__metadata"
            self.binding_parameter_tag_name = None
        else:
            self.collection_next_tag_name = "value"

    def get_media_type(self):
        return 'application/json;odata={0}'.format(self.metadata)

    @property
    def is_verbose(self):
        return self.metadata == ODataMetadataLevel.Verbose



from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.webs.web import Web


class WebCollection(BaseEntityCollection):
    """Web collection"""

    def __init__(self, context, resource_path=None, parent_web=None):
        """
        :type parent_web: Web
        """
        super(WebCollection, self).__init__(context, Web, resource_path, parent_web)

    def add(self, web_creation_information):
        """
        Create web site

        :type web_creation_information: office365.sharepoint.webs.web_creation_information.WebCreationInformation
        """
        target_web = Web(self.context)
        self.add_child(target_web)
        qry = ServiceOperationQuery(self, "add", None, web_creation_information, "parameters", target_web)
        self.context.add_query(qry)
        return target_web

    @property
    def resource_url(self):
        url = super(WebCollection, self).resource_url
        parent_web_url = self._parent.get_property("Url")
        if parent_web_url is not None:
            url = url.replace(self.context.service_root_url(), parent_web_url + '/_api/')
        return url



class BaseType:
    """Specifies the base type for a list."""

    None_ = -1
    GenericList = 0
    DocumentLibrary = 1
    Unused = 2
    DiscussionBoard = 3
    Survey = 4
    """Enumeration whose values specify a base type for a survey list."""
    Issue = 5



from office365.runtime.client_value import ClientValue


class Identity(ClientValue):
    """The Identity resource represents an identity of an actor. For example, an actor can be a user, device,
    or application. """

    def __init__(self):
        super(Identity, self).__init__()
        self.displayName = None
        self.id = None



from office365.runtime.client_object import ClientObject
from office365.runtime.queries.delete_entity_query import DeleteEntityQuery
from office365.runtime.queries.update_entity_query import UpdateEntityQuery
from office365.runtime.resource_path import ResourcePath


class Entity(ClientObject):
    """Base entity"""

    def update(self):
        """Updates the entity."""
        qry = UpdateEntityQuery(self)
        self.context.add_query(qry)
        return self

    def delete_object(self):
        """Deletes the entity."""
        qry = DeleteEntityQuery(self)
        self.context.add_query(qry)
        self.remove_from_parent_collection()
        return self

    @property
    def context(self):
        """
        :rtype: office365.graph_client.GraphClient
        """
        return self._context

    @property
    def entity_type_name(self):
        name = type(self).__name__
        return "microsoft.graph." + name[0].lower() + name[1:]

    @property
    def id(self):
        """The unique identifier of the entity.
        :rtype: str or None
        """
        return self.properties.get('id', None)

    def set_property(self, name, value, persist_changes=True):
        super(Entity, self).set_property(name, value, persist_changes)
        if name == "id" and self._resource_path is None:
            self._resource_path = ResourcePath(value, self._parent_collection.resource_path)
        return self



from office365.runtime.client_value import ClientValue


class ChangeToken(ClientValue):
    """Represents the unique sequential location of a change within the change log. Client applications can use the
    change token as a starting point for retrieving changes."""

    def __init__(self):
        super(ChangeToken, self).__init__()
        self.StringValue = None



class TokenResponse(object):

    def __init__(self, accessToken=None, tokenType=None, **kwargs):
        self.accessToken = accessToken
        self.tokenType = tokenType
        for key, value in kwargs.items():
            setattr(self, key, value)

    @property
    def is_valid(self):
        return self.accessToken is not None and self.tokenType == 'Bearer'

    @staticmethod
    def from_json(value):
        error = value.get('error', None)
        if error:
            raise ValueError(value)

        def _normalize_key(name):
            key_parts = name.split("_")
            if len(key_parts) >= 2:
                names = [n.title() for n in key_parts[1:]]
                return key_parts[0] + "".join(names)
            return name
        json = {_normalize_key(k): v for k, v in value.items()}
        return TokenResponse(**json)



from office365.outlook.calendar.emailAddress import EmailAddress
from office365.runtime.client_value import ClientValue


class Recipient(ClientValue):
    """Represents information about a user in the sending or receiving end of an event, message or group post."""

    def __init__(self, email_address=EmailAddress()):
        """

        :param EmailAddress email_address: The recipient's email address.
        """
        super(Recipient, self).__init__()
        self.emailAddress = email_address

    @staticmethod
    def from_email(value):
        """
        :type value: str or EmailAddress
        """
        if isinstance(value, EmailAddress):
            return Recipient(value)
        else:
            return Recipient(EmailAddress(value))



from office365.runtime.client_value_collection import ClientValueCollection
from office365.sharepoint.fields.field_lookup_value import FieldLookupValue


class FieldMultiLookupValue(ClientValueCollection):

    def __init__(self):
        super(FieldMultiLookupValue, self).__init__(FieldLookupValue)



from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.files.checkedOutFile import CheckedOutFile


class CheckedOutFileCollection(BaseEntityCollection):
    def __init__(self, context, resource_path=None):
        super(CheckedOutFileCollection, self).__init__(context, CheckedOutFile, resource_path)



from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.sharepoint.base_entity import BaseEntity


class TenantSettings(BaseEntity):

    @staticmethod
    def current(context):
        """

        :type context: ClientContext
        :return: TenantSettings
        """
        settings = TenantSettings(context)
        qry = ServiceOperationQuery(settings, "Current", None, None, None, settings)
        qry.static = True
        context.add_query(qry)
        return settings



import copy

from office365.runtime.client_value import ClientValue
from office365.runtime.odata.json_light_format import JsonLightFormat
from office365.runtime.odata.odata_json_format import ODataJsonFormat
from office365.runtime.odata.odata_query_options import QueryOptions


class ClientObject(object):

    def __init__(self, context, resource_path=None, parent_collection=None, namespace=None):
        """
        Base client object which define named properties and relationships of an entity

        :type parent_collection: office365.runtime.client_object_collection.ClientObjectCollection or None
        :type resource_path: office365.runtime.resource_path.ResourcePath or None
        :type context: office365.runtime.client_runtime_context.ClientRuntimeContext
        :type namespace: str
        """
        self._properties = {}
        self._properties_metadata = {}
        self._entity_type_name = None
        self._query_options = QueryOptions()
        self._parent_collection = parent_collection
        self._context = context
        self._resource_path = resource_path
        self._namespace = namespace

    def set_metadata(self, name, group, value):
        if name not in self._properties_metadata:
            self._properties_metadata[name] = {}
        self._properties_metadata[name][group] = value

    def get_metadata(self, name, group, default_value=None):
        return self._properties_metadata.get(name, {}).get(group, default_value)

    def clear(self):
        self._properties_metadata = {}

    def execute_query(self):
        self.context.execute_query()
        return self

    def execute_query_retry(self, max_retry=5,
                            timeout_secs=5,
                            success_callback=None,
                            failure_callback=None):
        self.context.execute_query_retry(max_retry=max_retry,
                                         timeout_secs=timeout_secs,
                                         success_callback=success_callback,
                                         failure_callback=failure_callback)
        return self

    def build_request(self):
        return self.context.build_request()

    def get(self):
        self.context.load(self)
        return self

    def is_property_available(self, name):
        """Returns a Boolean value that indicates whether the specified property has been retrieved or set.

        :param str name: A Property name
        """
        if name in self.properties:
            return True
        return False

    def expand(self, names):
        """

        :type names: list[str]
        """
        self.query_options.expand = names
        return self

    def select(self, names):
        """

        :param list[str] names:
        :return:
        """
        self.query_options.select = names
        return self

    def remove_from_parent_collection(self):
        if self._parent_collection is None:
            return
        self._parent_collection.remove_child(self)

    def get_property(self, name, default_value=None):
        """
        Gets property value

        :param str name: property name
        :param any default_value: property value
        """
        if default_value is None:
            normalized_name = name[0].lower() + name[1:]
            default_value = getattr(self, normalized_name, None)
        return self._properties.get(name, default_value)

    def set_property(self, name, value, persist_changes=True):
        """Sets property value

        :param str name: Property name
        :param any value: Property value
        :param bool persist_changes: Persist changes
        """
        self._properties_metadata[name] = {}
        if persist_changes:
            self.set_metadata(name, "persist", True)

        prop_type = self.get_property(name)
        if isinstance(prop_type, ClientObject) or isinstance(prop_type, ClientValue) and value is not None:
            if isinstance(value, list):
                [prop_type.set_property(i, v, persist_changes) for i, v in enumerate(value)]
                self._properties[name] = prop_type
            elif isinstance(value, dict):
                [prop_type.set_property(k, v, persist_changes) for k, v in value.items()]
                self._properties[name] = prop_type
            else:
                self._properties[name] = value
        else:
            self._properties[name] = value
        return self

    def ensure_property(self, name, action, *args, **kwargs):
        """
        Ensures if property is loaded

        :type action: () -> None
        :type name: str
        """
        return self.ensure_properties([name], action, *args, **kwargs)

    def ensure_properties(self, names, action, *args, **kwargs):
        """
        Ensure if list of properties are loaded

        :type action: (any) -> None
        :type names: str or list[str]
        """

        names_to_include = [n for n in names if not self.is_property_available(n)]
        if len(names_to_include) > 0:
            from office365.runtime.queries.read_entity_query import ReadEntityQuery
            qry = ReadEntityQuery(self, names_to_include)
            self.context.add_query(qry, execute_first=True, set_as_current=False)
            self.context.after_query_execute(qry, action, *args, **kwargs)
        else:
            action(*args, **kwargs)
        return self

    def clone_object(self):
        result = copy.deepcopy(self)
        result._context = self.context
        return result

    @property
    def entity_type_name(self):
        if self._entity_type_name is None:
            if self._namespace is None:
                self._entity_type_name = type(self).__name__
            else:
                self._entity_type_name = ".".join([self._namespace, type(self).__name__])
        return self._entity_type_name

    @property
    def resource_url(self):
        """Generate resource Url

        :rtype: str or None
        """
        if self.resource_path:
            url = self.context.service_root_url() + self.resource_path.to_url()
            if not self.query_options.is_empty:
                url = url + "?" + self._query_options.to_url()
            return url
        return None

    @property
    def context(self):
        return self._context

    @property
    def resource_path(self):
        return self._resource_path

    @property
    def query_options(self):
        return self._query_options

    @property
    def properties(self):
        return self._properties

    @property
    def parent_collection(self):
        return self._parent_collection

    def to_json(self, json_format=None):
        """
        :type json_format: office365.runtime.odata.odata_json_format.ODataJsonFormat or None
        """
        ser_prop_names = [n for n, p in self._properties_metadata.items() if p.get("persist", False) is True]
        json = dict((k, self.get_property(k)) for k in self.properties if k in ser_prop_names)
        for k, v in json.items():
            if isinstance(v, ClientObject) or isinstance(v, ClientValue):
                json[k] = v.to_json(json_format)

        if json and self.entity_type_name is not None:
            if isinstance(json_format, JsonLightFormat) and json_format.is_verbose:
                json[json_format.metadata_type_tag_name] = {'type': self.entity_type_name}
            elif isinstance(json_format, ODataJsonFormat) and json_format.metadata == "minimal":
                json[json_format.metadata_type_tag_name] = "#" + self.entity_type_name
        return json



from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.forms.form import Form


class FormCollection(BaseEntityCollection):

    def __init__(self, context, resource_path=None):
        super(FormCollection, self).__init__(context, Form, resource_path)

    def get_by_page_type(self):
        pass



from office365.directory import DirectoryObject


class DirectoryRoleTemplate(DirectoryObject):
    """Represents a directory role template. A directory role template specifies the property values of a directory
    role (directoryRole). """



from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.permissions.roleDefinitionCollection import RoleDefinitionCollection
from office365.sharepoint.principal.principal import Principal


class RoleAssignment(BaseEntity):
    """An association between a principal or a site group and a role definition."""

    @property
    def principalId(self):
        """"""
        return self.properties.get("PrincipalId", None)

    @property
    def member(self):
        """Specifies the user or group corresponding to the role assignment."""
        return self.properties.get("Member",
                                   Principal(self.context, ResourcePath("Member", self.resource_path)))

    @property
    def roleDefinitionBindings(self):
        """Specifies a collection of role definitions for this role assignment."""
        return self.properties.get("RoleDefinitionBindings",
                                   RoleDefinitionCollection(self.context,
                                                            ResourcePath("RoleDefinitionBindings",
                                                                         self.resource_path)))



from office365.base_item import BaseItem
from office365.onedrive.columns.calculated_column import CalculatedColumn
from office365.onedrive.columns.choice_column import ChoiceColumn
from office365.onedrive.columns.default_column_value import DefaultColumnValue


class ColumnDefinition(BaseItem):

    @property
    def indexed(self):
        """Specifies whether the column values can used for sorting and searching."""
        return self.properties.get('indexed', None)

    @property
    def column_group(self):
        """For site columns, the name of the group this column belongs to. Helps organize related columns."""
        return self.properties.get('columnGroup', None)

    @property
    def calculated(self):
        """This column's data is calculated based on other columns."""
        return self.get_property('calculated', CalculatedColumn())

    @property
    def choice(self):
        """This column stores data from a list of choices."""
        return self.get_property('choice', ChoiceColumn())

    @property
    def default_value(self):
        """The default value for this column."""
        return self.get_property('defaultValue', DefaultColumnValue())



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity


class LogExport(BaseEntity):

    def __init__(self, context):
        """This is the primary class that should be instantiated to obtain metadata about the
        logs that you can download."""
        super(LogExport, self).__init__(context, ResourcePath("Microsoft.Online.SharePoint.SPLogger.LogExport"))

    def get_files(self, partitionId, logType):
        pass

    def get_log_types(self):
        result = ClientResult(self.context)
        qry = ServiceOperationQuery(self, "GetLogTypes")
        self.context.add_query(qry)
        return result



from office365.runtime.client_value import ClientValue


class SPSiteCreationResponse(ClientValue):

    def __init__(self):
        super(SPSiteCreationResponse, self).__init__()
        self.SiteId = None
        self.SiteStatus = None
        self.SiteUrl = None



from office365.sharepoint.fields.field_lookup import FieldLookup


class TaxonomyField(FieldLookup):
    """Represents a taxonomy field."""

    @property
    def anchor_id(self):
        """Gets or sets the GUID of the anchor Term object for a TaxonomyField object."""
        return self.properties.get('AnchorId', None)

    @property
    def is_anchor_valid(self):
        """Gets a Boolean value that specifies whether the Term object identified by the AnchorId property is valid."""
        return self.properties.get('IsAnchorValid', None)

    @property
    def text_field(self):
        """Gets the GUID that identifies the hidden text field in an item."""
        return self.properties.get('TextField', None)

    @text_field.setter
    def text_field(self, value):
        """Sets the GUID that identifies the hidden text field in an item."""
        self.set_property("TextField", value)



from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.sharepoint.webparts.client_web_part import ClientWebPart


class ClientWebPartCollection(BaseEntityCollection):
    """Web collection"""

    def __init__(self, context, resource_path=None):
        super(ClientWebPartCollection, self).__init__(context, ClientWebPart, resource_path)



class ClientQuery(object):
    """Client query"""

    def __init__(self, context, binding_type=None, parameter_type=None, parameter_name=None, return_type=None):
        """
        Base query

        :type context: office365.runtime.client_runtime_context.ClientRuntimeContext
        :type binding_type: office365.runtime.client_object.ClientObject or None
        :type parameter_type:  office365.runtime.client_object.ClientObject or ClientValue or dict or bytes or None
        :type parameter_name:  str or None
        :type return_type:  office365.runtime.client_object.ClientObject or office365.runtime.client_result.ClientResult
           or office365.runtime.client_value.ClientValue or None
        """
        self._context = context
        self._binding_type = binding_type
        self._parameter_type = parameter_type
        self._parameter_name = parameter_name
        self._return_type = return_type

    def build_url(self):
        return self.binding_type.resource_url

    def build_request(self):
        return self.context.build_single_request(self)

    def execute_query(self):
        self.context.execute_query()
        return self.return_type

    @property
    def context(self):
        return self._context

    @property
    def id(self):
        return id(self)

    @property
    def binding_type(self):
        return self._binding_type

    @property
    def parameter_name(self):
        return self._parameter_name

    @property
    def parameter_type(self):
        if isinstance(self._parameter_type, dict):
            return {k: v for k, v in self._parameter_type.items() if v is not None}
        else:
            return self._parameter_type

    @property
    def return_type(self):
        return self._return_type



from office365.runtime.http.http_method import HttpMethod


class RequestOptions(object):
    """Request options"""

    def __init__(self, url):
        """

        :type url: str
        """
        self.url = url
        self.data = None
        self.headers = {}
        self.auth = None
        self.method = HttpMethod.Get
        self.verify = True
        self.stream = False
        self.proxies = None

    @property
    def is_file(self):
        return hasattr(self.data, 'read') and callable(self.data.read)

    @property
    def is_bytes(self):
        return hasattr(self.data, 'decode') and callable(self.data.decode)

    def set_header(self, name, value):
        self.headers[name] = value

    def ensure_header(self, name, value):
        if name not in self.headers:
            self.headers[name] = value



from office365.runtime.client_result import ClientResult
from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity


class SPHSite(BaseEntity):

    def __init__(self, context):
        super(SPHSite, self).__init__(context, ResourcePath("SP.SPHSite"))

    @staticmethod
    def is_valid_home_site(context, site_url, return_value=None):
        """

        :param ClientResult return_value:
        :param office365.sharepoint.client_context.ClientContext context:
        :param str site_url:
        :return:
        """

        if return_value is None:
            return_value = ClientResult(context)
        sph = SPHSite(context)
        params = {"siteUrl": site_url}
        qry = ServiceOperationQuery(sph, "IsValidHomeSite", params, None, None, return_value)
        qry.static = True
        context.add_query(qry)
        return return_value

    @staticmethod
    def set_as_home_site(context, site_url, return_value=None):
        """

        :param ClientResult return_value:
        :param office365.sharepoint.client_context.ClientContext context:
        :param str site_url:
        """

        if return_value is None:
            return_value = ClientResult(context)
        sph = SPHSite(context)
        params = {"siteUrl": site_url}
        qry = ServiceOperationQuery(sph, "SetSPHSite", params, None, None, return_value)
        qry.static = True
        context.add_query(qry)
        return return_value



from office365.runtime.client_value import ClientValue


class SharingLinkInfo(ClientValue):

    def __init__(self):
        """
        Specifies the information about the tokenized sharing link.

        """
        super(SharingLinkInfo, self).__init__()
        self.AllowsAnonymousAccess = None
        self.ApplicationId = None
        self.CreatedBy = None
        self.PasswordProtected = None



from office365.runtime.client_value import ClientValue


class SiteCollectionType(ClientValue):
    """The siteCollection resource provides more information about a site collection. """

    def __init__(self):
        super(SiteCollectionType, self).__init__()
        self._hostname = None
        self._root = None

    @property
    def hostname(self):
        """The hostname for the site collection."""
        return self._hostname

    @property
    def root(self):
        """If present, indicates that this is a root site collection in SharePoint."""
        return self._root



class UserCredential(object):

    def __init__(self, user_name, password):
        """

        :type password: str
        :type user_name str
        """
        self.userName = user_name
        self.password = password



class ODataType(object):

    def __init__(self):
        self.name = None
        self.namespace = None
        self.baseType = None
        self.properties = {}
        self.methods = {}

    def add_property(self, prop_schema):
        """
        :type prop_schema:  office365.runtime.odata.odata_property.ODataProperty
        """
        alias = prop_schema.name
        #if type_schema['state'] == 'detached':
        #    prop_schema['state'] = 'detached'
        #else:
        #    prop_schema['state'] = 'attached'
        #type_alias = type_schema['name']
        self.properties[alias] = prop_schema



from office365.entity import Entity
from office365.entity_collection import EntityCollection
from office365.runtime.client_result import ClientResult
from office365.runtime.client_value_collection import ClientValueCollection
from office365.runtime.queries.service_operation_query import ServiceOperationQuery


class DirectoryObject(Entity):
    """Represents an Azure Active Directory object. The directoryObject type is the base type for many other
    directory entity types. """

    def get_member_objects(self, security_enabled_only=True):
        """Returns all the groups and directory roles that a user, group, or directory object is a member of.
        This function is transitive.

        :type security_enabled_only: bool"""
        result = ClientResult(self.context, ClientValueCollection(str))
        payload = {
            "securityEnabledOnly": security_enabled_only
        }
        qry = ServiceOperationQuery(self, "getMemberObjects", None, payload, None, result)
        self.context.add_query(qry)
        return result

    def get_member_groups(self, security_enabled_only=True):
        """Return all the groups that the specified user, group, or directory object is a member of. This function is
        transitive.

        :type security_enabled_only: bool"""
        result = ClientResult(self.context)
        payload = {
            "securityEnabledOnly": security_enabled_only
        }
        qry = ServiceOperationQuery(self, "getMemberGroups", None, payload, None, result)
        self.context.add_query(qry)
        return result

    def get_available_extension_properties(self, is_synced_from_on_premises=None):
        """
        Return all or a filtered list of the directory extension properties that have been registered in a directory.
        The following entities support extension properties: user, group, organization, device, application,
        and servicePrincipal.
        """
        from office365.directory.extensions.extension_property import ExtensionProperty
        return_type = EntityCollection(self.context, ExtensionProperty)
        payload = {
            "isSyncedFromOnPremises": is_synced_from_on_premises
        }
        qry = ServiceOperationQuery(self, "getAvailableExtensionProperties", None, payload, None, return_type)
        self.context.add_query(qry)
        return return_type

    @property
    def deleted_datetime(self):
        """ETag for the item."""
        return self.properties.get('deletedDateTime', None)

    def set_property(self, name, value, persist_changes=True):
        super(DirectoryObject, self).set_property(name, value, persist_changes)



from office365.runtime.client_result import ClientResult
from office365.runtime.http.http_method import HttpMethod
from office365.runtime.queries.service_operation_query import ServiceOperationQuery


def create_search_query(drive_item, query_text):
    """

    :param office365.onedrive.driveItem.DriveItem drive_item:
    :param str query_text:
    """
    result = ClientResult(drive_item.context)
    qry = ServiceOperationQuery(drive_item, "search", {"q": query_text}, None, None, result)

    def _construct_query(request):
        """
        :type request: office365.runtime.http.request_options.RequestOptions
        """
        request.method = HttpMethod.Get
    drive_item.context.before_execute(_construct_query)
    return qry



from office365.entity import Entity
from office365.runtime.resource_path import ResourcePath
from office365.teams.apps.teams_app import TeamsApp
from office365.teams.tabs.teams_tab_configuration import TeamsTabConfiguration


class TeamsTab(Entity):
    """
    A teamsTab is a tab that's pinned (attached) to a channel within a team.
    """

    @property
    def teams_app(self):
        """The application that is linked to the tab. This cannot be changed after tab creation."""
        return self.properties.get('teamsApp',
                                   TeamsApp(self.context, ResourcePath("teamsApp", self.resource_path)))

    @property
    def configuration(self):
        """
        Container for custom settings applied to a tab. The tab is considered configured only once this property is set.
        :rtype: TeamsTabConfiguration
        """
        return self.properties.get("configuration", TeamsTabConfiguration())



from office365.entity import Entity
from office365.entity_collection import EntityCollection
from office365.onedrive.itemActivity import ItemActivity
from office365.runtime.resource_path import ResourcePath


class ItemActivityStat(Entity):
    """The itemActivityStat resource provides information about activities that took place
    within an interval of time."""

    @property
    def activities(self):
        """Exposes the itemActivities represented in this itemActivityStat resource."""
        return self.properties.get('activities',
                                   EntityCollection(self.context, ItemActivity,
                                                    ResourcePath("activities", self.resource_path)))



from office365.runtime.client_value import ClientValue


class RecycleBinQueryInformation(ClientValue):

    def __init__(self, is_ascending, item_state, order_by, paging_info, row_limit, show_only_my_items):
        """

        :type show_only_my_items: bool
        :type row_limit: int
        :type paging_info: str
        :type order_by: int
        :type item_state: int
        :type is_ascending: bool
        """
        super(RecycleBinQueryInformation, self).__init__()
        self.IsAscending = is_ascending
        self.ItemState = item_state
        self.OrderBy = order_by
        self.PagingInfo = paging_info
        self.RowLimit = row_limit
        self.ShowOnlyMyItems = show_only_my_items



from office365.runtime.client_value import ClientValue
from office365.sharepoint.search.simpleDataTable import SimpleDataTable


class RelevantResults(ClientValue):

    def __init__(self):
        super(RelevantResults, self).__init__()
        self.GroupTemplateId = None
        self.ItemTemplateId = None
        self.Properties = []
        self.ResultTitle = None
        self.ResultTitleUrl = None
        self.RowCount = None
        self.Table = SimpleDataTable()
        self.TotalRows = None
        self.TotalRowsIncludingDuplicates = None



from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.alerts.alert_collection import AlertCollection
from office365.sharepoint.principal.principal import Principal
from office365.sharepoint.principal.user_id_info import UserIdInfo


class User(Principal):
    """Represents a user in Microsoft SharePoint Foundation. A user is a type of SP.Principal."""

    def get(self):
        """
        :rtype: User
        """
        return super(User, self).get()

    @property
    def groups(self):
        """Gets a collection of group objects that represents all of the groups for the user."""
        from office365.sharepoint.principal.group_collection import GroupCollection
        return self.properties.get('Groups',
                                   GroupCollection(self.context, ResourcePath("Groups", self.resource_path)))

    @property
    def alerts(self):
        return self.properties.get('Alerts',
                                   AlertCollection(self.context,
                                                   ResourcePath("Alerts", self.resource_path)))

    @property
    def is_site_admin(self):
        """Gets or sets a Boolean value that specifies whether the user is a site collection administrator."""
        return self.properties.get('isSiteAdmin', None)

    @property
    def user_id(self):
        """Gets the information of the user that contains the user's name identifier and the issuer of the
         user's name identifier."""
        return self.properties.get('UserId', UserIdInfo())

    def expire(self):
        qry = ServiceOperationQuery(self, "Expire")
        self.context.add_query(qry)
        return self



from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.taxonomy.taxonomy_item import TaxonomyItem, TaxonomyItemCollection
from office365.sharepoint.taxonomy.term_set import TermSet


class TermGroup(TaxonomyItem):
    """Represents the top-level container in a TermStore object."""

    @property
    def term_sets(self):
        return self.properties.get("termSets",
                                   TaxonomyItemCollection(self.context, TermSet,
                                                          ResourcePath("termSets", self.resource_path)))




class PermissionKind:
    """Specifies permissions that are used to define user roles."""
    def __init__(self):
        pass

    EmptyMask = 0
    ViewListItems = 1
    AddListItems = 2
    EditListItems = 3
    DeleteListItems = 4
    ApproveItems = 5
    OpenItems = 6
    ViewVersions = 7
    DeleteVersions = 8
    CancelCheckout = 9
    ManagePersonalViews = 10
    ManageLists = 12
    ViewFormPages = 13
    AnonymousSearchAccessList = 14
    Open = 17
    ViewPages = 18
    AddAndCustomizePages = 19
    ApplyThemeAndBorder = 20
    ApplyStyleSheets = 21
    ViewUsageData = 22
    CreateSSCSite = 23
    ManageSubwebs = 24
    CreateGroups = 25
    ManagePermissions = 26
    BrowseDirectories = 27
    BrowseUserInfo = 28
    AddDelPrivateWebParts = 29
    UpdatePersonalWebParts = 30
    ManageWeb = 31
    AnonymousSearchAccessWebLists = 32
    UseClientIntegration = 37
    UseRemoteAPIs = 38
    ManageAlerts = 39
    CreateAlerts = 40
    EditMyUserInfo = 41
    EnumeratePermissions = 63
    FullMask = 65



from office365.runtime.client_value import ClientValue


class SignInStatus(ClientValue):
    """Provides the sign-in status (Success or Failure) of the sign-in."""
    def __init__(self, additional_details=None, error_code=None, failure_reason=None):
        super(SignInStatus, self).__init__()
        self.additionalDetails = additional_details
        self.errorCode = error_code
        self.failureReason = failure_reason



from office365.entity_collection import EntityCollection
from office365.teams.channels.channel import Channel


class ChannelCollection(EntityCollection):
    """Team's collection"""

    def __init__(self, context, resource_path=None):
        super(ChannelCollection, self).__init__(context, Channel, resource_path)

    def add(self, display_name, description=None):
        """Create a new channel in a Microsoft Team, as specified in the request body.

        :param str description: Optional textual description for the channel.
        :param str display_name: Channel name as it will appear to the user in Microsoft Teams.
        :rtype: Channel
        """
        return super(ChannelCollection, self).add(
            displayName=display_name,
            description=description
        )



from office365.runtime.queries.service_operation_query import ServiceOperationQuery
from office365.runtime.resource_path import ResourcePath
from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.sharing.userDirectoryInfo import UserDirectoryInfo


class SharingUtility(BaseEntity):

    def __init__(self, context):
        super(SharingUtility, self).__init__(context, ResourcePath("SharingUtility"))

    @staticmethod
    def get_user_directory_info_by_email(context, email):
        """

        :param str email:
        :param office365.sharepoint.client_context.ClientContext context:
        """
        result = UserDirectoryInfo()
        payload = {
            "email": email
        }
        utility = SharingUtility(context)
        qry = ServiceOperationQuery(utility, "GetUserDirectoryInfoByEmail", None, payload, None, result)
        qry.static = True
        context.add_query(qry)
        return result



from office365.runtime.client_value import ClientValue


class UploadSession(ClientValue):
    """The UploadSession resource provides information about how to upload large files to OneDrive, OneDrive for
    Business, or SharePoint document libraries. """

    def __init__(self):
        super(UploadSession, self).__init__()
        self.expirationDateTime = None
        self.nextExpectedRanges = None
        self.uploadUrl = None



from office365.runtime.client_value import ClientValue


class SystemFacet(ClientValue):
    """
    The System facet indicates that the object is managed by the system for its own operation.
    Most apps should ignore items that have a System facet.
    """
    pass



from office365.runtime.client_value import ClientValue


class ResourcePath(ClientValue):

    def __init__(self, decoded_url):
        super(ResourcePath, self).__init__()
        self.DecodedUrl = decoded_url

    @property
    def entity_type_name(self):
        return "SP.ResourcePath"

    def __str__(self):
        return "DecodedUrl='{0}'".format(self.DecodedUrl)



from office365.runtime.http.http_method import HttpMethod
from office365.runtime.queries.service_operation_query import ServiceOperationQuery


def create_download_file_query(source_file, file_object):
    """
    A download file content query

    :type source_file: office365.sharepoint.files.file.File
    :type file_object: typing.IO
    """
    qry = ServiceOperationQuery(source_file, "$value")

    def _process_response(response):
        """
        :type response: RequestOptions
        """
        file_object.write(response.content)

    def _construct_download_query(request):
        request.method = HttpMethod.Get
        source_file.context.after_execute(_process_response)

    source_file.context.before_execute(_construct_download_query)
    return qry



from office365.sharepoint.base_entity import BaseEntity
from office365.sharepoint.base_entity_collection import BaseEntityCollection


class SiteDesignPrincipal(BaseEntity):
    pass


class SiteDesignPrincipalCollection(BaseEntityCollection):

    def __init__(self, context, resource_path=None):
        super(SiteDesignPrincipalCollection, self).__init__(context, SiteDesignPrincipal, resource_path)



from office365.sharepoint.fields.field_lookup_value import FieldLookupValue


class FieldUserValue(FieldLookupValue):

    def __init__(self, user_id):
        """Represents the value of a user fields for a list item."""
        super(FieldUserValue, self).__init__(user_id)

    @staticmethod
    def from_user(user):
        """
        Initialize field value from User

        :param office365.sharepoint.principal.user.User user: User entity
        :return: FieldUserValue
        """
        value = FieldUserValue(-1)

        def _init_from_user():
            value.LookupId = user.id
        user.ensure_property("Id", _init_from_user)
        return value



class FileSystemObjectType:
    """Specifies the file system object type."""

    def __init__(self):
        pass

    Invalid = -1
    """Enumeration whose values specify whether the object is invalid. The value = -1."""

    File = 0
    """Enumeration whose values specify whether the object is a file. The value = 0."""

    Folder = 1
    """Enumeration whose values specify whether the object is a folder. The value = 1."""

    Web = 2
    """Enumeration whose values specify whether the object is a site. The values = 2."""



from office365.entity import Entity
from office365.runtime.client_value_collection import ClientValueCollection


class ConversationMember(Entity):
    """Represents a user in a team, a channel, or a chat. See also aadUserConversationMember."""

    @property
    def roles(self):
        """The roles for that user."""
        return self.properties.get('roles', ClientValueCollection(str))





from office365.sharepoint.base_entity_collection import BaseEntityCollection
from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
from office365.sharepoint.webs.web_template import WebTemplate


class WebTemplateCollection(BaseEntityCollection):
    """Specifies a collection of site templates."""

    def __init__(self, context, resource_path=None, parent=None):
        super(WebTemplateCollection, self).__init__(context, WebTemplate, resource_path, parent)

    def get_by_name(self, name):
        """Returns the SP.WebTemplate (section 3.2.5.151) specified by its name.<162>

        :param str name: The name of the SP.WebTemplate that is returned.

        """
        return WebTemplate(self.context,
                           ResourcePathServiceOperation("getByName", ["{name}".format(name=name)], self.resource_path))



from office365.entity import Entity


class MessageRule(Entity):
    """A rule that applies to messages in the Inbox of a user.

    In Outlook, you can set up rules for incoming messages in the Inbox to carry out specific actions
    upon certain conditions."""
    pass



from office365.entity import Entity


class ItemAnalytics(Entity):
    """The itemAnalytics resource provides analytics about activities that took place on an item.
    This resource is currently only available on SharePoint and OneDrive for Business."""
    pass
